@@ -45,7 +45,7 @@ static HHOOK hhookWnd = 0;
 HardHook::HardHook() {
 	int i;
 	baseptr = NULL;
-	for(i=0;i<5;i++)
+	for (i=0;i<5;i++)
 		orig[i]=replace[i]=0;
 }
 
@@ -67,7 +67,7 @@ void HardHook::setup(voidFunc func, voidFunc replacement) {
 	replace[0] = 0xe9;
 
 	if (VirtualProtect(fptr, 5, PAGE_EXECUTE_READ, &oldProtect)) {
-		for(i=0;i<5;i++)
+		for (i=0;i<5;i++)
 			orig[i]=fptr[i];
 		VirtualProtect(fptr, 5, oldProtect, &restoreProtect);
 		baseptr = fptr;
@@ -88,11 +88,14 @@ void HardHook::inject() {
 	if (! baseptr)
 		return;
 	if (VirtualProtect(baseptr, 5, PAGE_EXECUTE_READWRITE, &oldProtect)) {
-		for(i=0;i<5;i++)
+		for (i=0;i<5;i++)
 			baseptr[i] = replace[i];
 		VirtualProtect(baseptr, 5, oldProtect, &restoreProtect);
 		FlushInstructionCache(GetCurrentProcess(),baseptr, 5);
 	}
+	for (i=0;i<5;i++)
+		if (baseptr[i] != replace[i])
+			ods("HH: Injection failure at byte %d", i);
 }
 
 void HardHook::restore() {
@@ -102,54 +105,74 @@ void HardHook::restore() {
 	if (! baseptr)
 		return;
 	if (VirtualProtect(baseptr, 5, PAGE_EXECUTE_READWRITE, &oldProtect)) {
-		for(i=0;i<5;i++)
+		for (i=0;i<5;i++)
 			baseptr[i] = orig[i];
 		VirtualProtect(baseptr, 5, oldProtect, &restoreProtect);
 		FlushInstructionCache(GetCurrentProcess(),baseptr, 5);
 	}
 }
 
+void HardHook::print() {
+	ods("HH: %02x %02x %02x %02x %02x => %02x %02x %02x %02x %02x (%02x %02x %02x %02x %02x)",
+	    orig[0], orig[1], orig[2], orig[3], orig[4],
+	    replace[0], replace[1], replace[2], replace[3], replace[4],
+	    baseptr[0], baseptr[1], baseptr[2], baseptr[3], baseptr[4]);
+}
+
 void __cdecl ods(const char *format, ...) {
-        if (!sm || ! sm->bDebug)
-                return;
+	if (!sm || ! sm->bDebug)
+		return;
 
-        char    buf[4096], *p = buf;
-        va_list args;
+	char    buf[4096], *p = buf;
+	va_list args;
 
-        va_start(args, format);
-        p += _vsnprintf(p, sizeof buf - 1, format, args);
-        va_end(args);
+	va_start(args, format);
+	int len = _vsnprintf_s(p, sizeof(buf) - 1, _TRUNCATE, format, args);
+	va_end(args);
 
-        while ( p > buf  &&  isspace(p[-1]) )
-                *--p = '\0';
+	if (len <= 0)
+		return;
 
-        *p++ = '\r';
-        *p++ = '\n';
-        *p   = '\0';
+	p += len;
 
-        OutputDebugStringA(buf);
-        FILE *f = fopen("c:\\overlay.log", "a");
-        fprintf(f, "%d %s", GetTickCount(), buf);
-        fclose(f);
+	while (p > buf  &&  isspace(p[-1]))
+		*--p = '\0';
+
+	*p++ = '\r';
+	*p++ = '\n';
+	*p   = '\0';
+
+	OutputDebugStringA(buf);
+	FILE *f = NULL;
+	errno_t res = fopen_s(&f, "c:\\overlay.log", "a");
+	if (f && (res == 0)) {
+		fprintf(f, "%d %s", GetTickCount(), buf);
+		fclose(f);
+	}
 }
 
 static LRESULT CALLBACK CallWndProc(int nCode, WPARAM wParam, LPARAM lParam) {
-	CWPSTRUCT *s = reinterpret_cast<CWPSTRUCT *>(lParam);
-	if (s) {
-		switch (s->message) {
-			case WM_CREATE:
-			case WM_DESTROY:
-			case WM_SETFOCUS:
-			case WM_GETMINMAXINFO:	// For things that link directly
-			case WM_GETICON:		// Worked for BF2
-			case WM_NCCREATE:		// Lots of games
-				checkD3D9Hook();
-				checkOpenGLHook();
-				checkOpenALHook();
-				// checkDSHook(s->hwnd);
-				break;
-			default:
-				break;
+	char procname[1024];
+	GetModuleFileName(NULL, procname, 1024);
+
+	if (strstr(procname, "mumble.exe") == NULL) {
+		CWPSTRUCT *s = reinterpret_cast<CWPSTRUCT *>(lParam);
+		if (s) {
+			switch (s->message) {
+				case WM_CREATE:
+				case WM_DESTROY:
+				case WM_SETFOCUS:
+				case WM_GETMINMAXINFO:	// For things that link directly
+				case WM_GETICON:		// Worked for BF2
+				case WM_NCCREATE:		// Lots of games
+					checkD3D9Hook();
+					checkOpenGLHook();
+					// checkOpenALHook();
+					// checkDSHook(s->hwnd);
+					break;
+				default:
+					break;
+			}
 		}
 	}
 	return CallNextHookEx(hhookWnd, nCode, wParam, lParam);
@@ -175,7 +198,7 @@ extern "C" __declspec(dllexport) void __cdecl InstallHooks() {
 	DWORD dwWaitResult = WaitForSingleObject(hHookMutex, 1000L);
 	if (dwWaitResult == WAIT_OBJECT_0) {
 		if (! sm->bHooked) {
-			GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (wchar_t *) &InstallHooks, &hSelf);
+			GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (char *) &InstallHooks, &hSelf);
 			if (hSelf == NULL) {
 				ods("Lib: Failed to find myself");
 			} else {
@@ -193,12 +216,19 @@ extern "C" __declspec(dllexport) SharedMem * __cdecl GetSharedMemory() {
 	return sm;
 }
 
-extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved) {
+extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
+
+	char procname[1024];
+	GetModuleFileName(NULL, procname, 1024);
+
+//	if ((strstr(procname, "mumble.exe") == NULL) && (strstr(procname, "Text3D.exe") == NULL) && (strstr(procname, "Lesson5.exe") == NULL))
+//		return TRUE;
+
 	switch (fdwReason) {
-		case DLL_PROCESS_ATTACH:
-			{
-				hSharedMutex = CreateMutex(NULL, false, L"MumbleSharedMutex");
-				hHookMutex = CreateMutex(NULL, false, L"MumbleHookMutex");
+		case DLL_PROCESS_ATTACH: {
+				ods("Lib: ProcAttach: %s", procname);
+				hSharedMutex = CreateMutex(NULL, false, "MumbleSharedMutex");
+				hHookMutex = CreateMutex(NULL, false, "MumbleHookMutex");
 				if ((hSharedMutex == NULL) || (hHookMutex == NULL)) {
 					ods("Lib: CreateMutex failed");
 					return FALSE;
@@ -210,7 +240,7 @@ extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvRe
 					return FALSE;
 				}
 
-				hMapObject = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0, sizeof(SharedMem), L"MumbleSharedMemory");
+				hMapObject = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0, sizeof(SharedMem), "MumbleSharedMemory");
 				if (hMapObject == NULL) {
 					ods("Lib: CreateFileMapping failed");
 					ReleaseMutex(hSharedMutex);
@@ -242,8 +272,7 @@ extern "C" BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvRe
 				ReleaseMutex(hSharedMutex);
 			}
 			break;
-		case DLL_PROCESS_DETACH:
-			{
+		case DLL_PROCESS_DETACH: {
 				UnmapViewOfFile(sm);
 				CloseHandle(hMapObject);
 				CloseHandle(hSharedMutex);
