@@ -29,47 +29,94 @@
 */
 
 #include "lib.h"
-#include <GL/gl.h>
-#include <GL/glext.h>
 
-#define INJDEF(name) static __typeof__(&name) o##name = NULL; static HardHook hh##name
-#define INJECT(name) ({ o##name = reinterpret_cast<__typeof__(&name)>(GetProcAddress(hGL, #name)); if (o##name) { hh##name.setup(reinterpret_cast<voidFunc>(o##name), reinterpret_cast<voidFunc>(my##name)); hh##name.inject(); } else { ods("OpenAL: No GetProc for %s", #name);} })
+typedef unsigned int    GLenum;
+typedef unsigned char   GLboolean;
+typedef void            GLvoid;
+typedef int             GLint;
+typedef unsigned char   GLubyte;
+typedef unsigned int    GLuint;
+typedef int             GLsizei;
+typedef float           GLfloat;
+typedef double          GLdouble;
 
-#define GLDEF(name) static __typeof__(&name) o##name = NULL;
+#define GL_TEXTURE_2D                           0x0DE1
+#define GL_UNSIGNED_BYTE                        0x1401
+#define GL_QUADS                                0x0007
+#define GL_RGBA                                 0x1908
+#define GL_BGRA                                 0x80E1
+#define GL_TEXTURE_WRAP_S                       0x2802
+#define GL_TEXTURE_WRAP_T                       0x2803
+#define GL_CLAMP_TO_EDGE                        0x812F
+#define GL_TEXTURE_MAG_FILTER                   0x2800
+#define GL_TEXTURE_MIN_FILTER                   0x2801
+#define GL_LINEAR                               0x2601
+#define GL_FRONT_AND_BACK                       0x0408
+#define GL_AMBIENT_AND_DIFFUSE                  0x1602
+#define GL_SRC_ALPHA                            0x0302
+#define GL_ONE_MINUS_SRC_ALPHA                  0x0303
+#define GL_TEXTURE_GEN_S                        0x0C60
+#define GL_TEXTURE_GEN_T                        0x0C61
+#define GL_TEXTURE_GEN_R                        0x0C62
+#define GL_TEXTURE_GEN_Q                        0x0C63
+#define GL_STENCIL_TEST                         0x0B90
+#define GL_SCISSOR_TEST                         0x0C11
+#define GL_SEPARABLE_2D                         0x8012
+#define GL_MINMAX                               0x802E
+#define GL_LIGHTING                             0x0B50
+#define GL_INDEX_LOGIC_OP                       0x0BF1
+#define GL_HISTOGRAM                            0x8024
+#define GL_FOG                                  0x0B60
+#define GL_DITHER                               0x0BD0
+#define GL_DEPTH_TEST                           0x0B71
+#define GL_CULL_FACE                            0x0B44
+#define GL_CONVOLUTION_1D                       0x8010
+#define GL_CONVOLUTION_2D                       0x8011
+#define GL_COLOR_TABLE                          0x80D0
+#define GL_COLOR_MATERIAL                       0x0B57
+#define GL_AUTO_NORMAL                          0x0D80
+#define GL_ALPHA_TEST                           0x0BC0
+#define GL_COLOR_LOGIC_OP                       0x0BF2
+#define GL_MATRIX_MODE                          0x0BA0
+#define GL_MODELVIEW                            0x1700
+#define GL_PROJECTION                           0x1701
+#define GL_BLEND                                0x0BE2
 
-// #define RESOLVE(name) o##name = reinterpret_cast<__typeof__(&name)>(GetProcAddress(hGL, #name))
+#define TDEF(ret, name, arg) typedef ret (__stdcall * t##name) arg
+#define GLDEF(ret, name, arg) TDEF(ret, name, arg); t##name o##name = NULL
 
-extern "C" WINGDIAPI BOOL APIENTRY wglSwapBuffers(HDC);
+GLDEF(HGLRC, wglCreateContext, (HDC));
+GLDEF(void, glGenTextures, (GLsizei, GLuint *));
+GLDEF(void, glEnable, (GLenum));
+GLDEF(void, glDisable, (GLenum));
+GLDEF(void, glBlendFunc, (GLenum, GLenum));
+GLDEF(void, glColorMaterial, (GLenum, GLenum));
+GLDEF(void, glViewport, (GLint, GLint, GLsizei, GLsizei));
+GLDEF(void, glMatrixMode, (GLenum));
+GLDEF(void, glLoadIdentity, (void));
+GLDEF(void, glOrtho, (GLdouble, GLdouble, GLdouble, GLdouble, GLdouble, GLdouble));
+GLDEF(void, glBindTexture, (GLenum, GLuint));
+GLDEF(void, glPushMatrix, (void));
+GLDEF(void, glColor4ub, (GLubyte, GLubyte, GLubyte, GLubyte));
+GLDEF(void, glTranslatef, (GLfloat, GLfloat, GLfloat));
+GLDEF(void, glBegin, (GLenum));
+GLDEF(void, glEnd, (void));
+GLDEF(void, glTexCoord2f, (GLfloat, GLfloat));
+GLDEF(void, glVertex2f, (GLfloat, GLfloat));
+GLDEF(void, glPopMatrix, (void));
+GLDEF(void, glTexParameteri, (GLenum, GLenum, GLint));
+GLDEF(void, glTexImage2D, (GLenum, GLint, GLint, GLsizei, GLsizei, GLint, GLenum, GLenum, const GLvoid *));
+GLDEF(void, wglMakeCurrent, (HDC, HGLRC));
+GLDEF(HGLRC, wglGetCurrentContext, (void));
+GLDEF(HDC, wglGetCurrentDC, (void));
+GLDEF(int, GetDeviceCaps, (HDC, int));
 
-INJDEF(wglSwapBuffers);
-INJDEF(wglSwapLayerBuffers);
-INJDEF(SwapBuffers);
+#define INJDEF(ret, name, arg) GLDEF(ret, name, arg); static HardHook hh##name
+#define INJECT(name) { o##name = reinterpret_cast<t##name>(GetProcAddress(hGL, #name)); if (o##name) { hh##name.setup(reinterpret_cast<voidFunc>(o##name), reinterpret_cast<voidFunc>(my##name)); hh##name.inject(); } else { ods("OpenGL: No GetProc for %s", #name);} }
 
-GLDEF(wglCreateContext);
-GLDEF(glGenTextures);
-GLDEF(glEnable);
-GLDEF(glDisable);
-GLDEF(glBlendFunc);
-GLDEF(glColorMaterial);
-GLDEF(glViewport);
-GLDEF(glMatrixMode);
-GLDEF(glLoadIdentity);
-GLDEF(glOrtho);
-GLDEF(glBindTexture);
-GLDEF(glPushMatrix);
-GLDEF(glColor4ub);
-GLDEF(glTranslatef);
-GLDEF(glBegin);
-GLDEF(glEnd);
-GLDEF(glTexCoord2f);
-GLDEF(glVertex2f);
-GLDEF(glPopMatrix);
-GLDEF(glTexParameteri);
-GLDEF(glTexImage2D);
-GLDEF(wglMakeCurrent);
-GLDEF(wglGetCurrentContext);
-GLDEF(wglGetCurrentDC);
-GLDEF(GetDeviceCaps);
+INJDEF(BOOL, wglSwapLayerBuffers, (HDC, UINT));
+INJDEF(BOOL, wglSwapBuffers, (HDC));
+INJDEF(BOOL, SwapBuffers, (HDC));
 
 static bool bHooked = false;
 static bool bChaining = false;
@@ -146,16 +193,16 @@ void Context::draw(HDC hdc) {
 
 	ods("DrawStart: Screen is %d x %d", width, height);
 
-	if (sm->fFontSize < 0.01)
-		sm->fFontSize = 0.01;
-	else if (sm->fFontSize > 1.0)
-		sm->fFontSize = 1.0;
+	if (sm->fFontSize < 0.01f)
+		sm->fFontSize = 0.01f;
+	else if (sm->fFontSize > 1.0f)
+		sm->fFontSize = 1.0f;
 
 	int iHeight = (int)((height * 1.0) * sm->fFontSize);
 	if (iHeight > TEXT_HEIGHT)
 		iHeight = TEXT_HEIGHT;
 
-	float s = iHeight / 60.0;
+	float s = iHeight / 60.0f;
 	int y = 0;
 	int idx = 0;
 
@@ -213,7 +260,7 @@ void Context::draw(HDC hdc) {
 
 	if (y < 1)
 		y = 1;
-	if ((y + h + 1) > height)
+	if ((y + h + 1) > (int)height)
 		y = height - h - 1;
 
 
@@ -229,7 +276,7 @@ void Context::draw(HDC hdc) {
 
 		if (x < 1)
 			x = 1;
-		if ((x + w + 1) > width)
+		if ((x + w + 1) > (int)width)
 			x = width - w - 1;
 
 		ods("OpenGL: Drawing text at %d %d  %d %d", x, y + yofs[i], w, iHeight);
@@ -237,26 +284,26 @@ void Context::draw(HDC hdc) {
 		oglPushMatrix();
 		oglLoadIdentity();
 
-		double xm = 0.0;
-		double ym = 0.0;
-		double xmx = (1.0 * widths[i]) / TEXT_WIDTH;
-		double ymx = 1.0;
+		float xm = 0.0;
+		float ym = 0.0;
+		float xmx = (1.0f * widths[i]) / TEXT_WIDTH;
+		float ymx = 1.0f;
 
 		unsigned int c = color[i];
 
 		oglColor4ub((c >> 16) & 0xFF, (c >> 8) & 0xFF, c & 0xFF, (c >> 24) & 0xFF);
 
 
-		oglTranslatef(x, y + yofs[i], 0.0);
+		oglTranslatef(static_cast<float>(x), static_cast<float>(y + yofs[i]), 0.0f);
 		oglBegin(GL_QUADS);
 		oglTexCoord2f(xm, ymx);
-		oglVertex2f(0, iHeight);
+		oglVertex2f(0.0f, static_cast<float>(iHeight));
 		oglTexCoord2f(xm, ym);
-		oglVertex2f(0, 0);
+		oglVertex2f(0.0f, 0.0f);
 		oglTexCoord2f(xmx, ym);
-		oglVertex2f(w, 0);
+		oglVertex2f(static_cast<float>(w), 0.0f);
 		oglTexCoord2f(xmx, ymx);
-		oglVertex2f(w, iHeight);
+		oglVertex2f(static_cast<float>(w), static_cast<float>(iHeight));
 		oglEnd();
 		oglPopMatrix();
 	}
@@ -282,7 +329,7 @@ static void doSwap(HDC hdc) {
 	owglMakeCurrent(oldhdc, oldctx);
 }
 
-static WINGDIAPI BOOL APIENTRY mywglSwapBuffers(HDC hdc) {
+static BOOL __stdcall mywglSwapBuffers(HDC hdc) {
 	ods("OpenGL: wglSwapBuffers");
 	doSwap(hdc);
 	hhwglSwapBuffers.restore();
@@ -291,7 +338,7 @@ static WINGDIAPI BOOL APIENTRY mywglSwapBuffers(HDC hdc) {
 	return ret;
 }
 
-static WINGDIAPI BOOL WINAPI mySwapBuffers(HDC hdc) {
+static BOOL __stdcall mySwapBuffers(HDC hdc) {
 	ods("OpenGL: SwapBuffers");
 	hhSwapBuffers.restore();
 	BOOL ret=oSwapBuffers(hdc);
@@ -299,7 +346,7 @@ static WINGDIAPI BOOL WINAPI mySwapBuffers(HDC hdc) {
 	return ret;
 }
 
-static WINGDIAPI BOOL WINAPI mywglSwapLayerBuffers(HDC hdc, UINT fuPlanes) {
+static BOOL __stdcall mywglSwapLayerBuffers(HDC hdc, UINT fuPlanes) {
 	ods("OpenGL: SwapLayerBuffers %x",fuPlanes);
 	hhwglSwapLayerBuffers.restore();
 	BOOL ret=owglSwapLayerBuffers(hdc, fuPlanes);
@@ -309,7 +356,7 @@ static WINGDIAPI BOOL WINAPI mywglSwapLayerBuffers(HDC hdc, UINT fuPlanes) {
 
 
 #undef GLDEF
-#define GLDEF(name) o##name = reinterpret_cast<__typeof__(&name)>(GetProcAddress(hGL, #name))
+#define GLDEF(name) o##name = reinterpret_cast<t##name>(GetProcAddress(hGL, #name))
 
 void checkOpenGLHook() {
 	if (bChaining) {
@@ -319,16 +366,16 @@ void checkOpenGLHook() {
 
 	bChaining = true;
 
-	HMODULE hGL = GetModuleHandle(L"OpenGL32.DLL");
+	HMODULE hGL = GetModuleHandle("OpenGL32.DLL");
 
 	if (hGL != NULL) {
 		if (! bHooked) {
-			wchar_t procname[1024];
+			char procname[1024];
 			GetModuleFileName(NULL, procname, 1024);
-			ods("OpenGL: CreateWnd in unhooked OpenGL App %ls", procname);
+			ods("OpenGL: CreateWnd in unhooked OpenGL App %s", procname);
 			bHooked = true;
 
-			if (wcsstr(procname, L"mumble.exe") != NULL) {
+			if (strstr(procname, "mumble.exe") != NULL) {
 				ods("OpenGL: Ignoring mumble.exe");
 			} else {
 				INJECT(wglSwapBuffers);
@@ -359,7 +406,7 @@ void checkOpenGLHook() {
 				GLDEF(wglGetCurrentContext);
 				GLDEF(wglGetCurrentDC);
 
-				hGL = GetModuleHandle(L"GDI32.DLL");
+				hGL = GetModuleHandle("GDI32.DLL");
 				if (hGL) {
 					// INJECT(SwapBuffers);
 					GLDEF(GetDeviceCaps);
