@@ -36,9 +36,6 @@
 
 #define NBLOCKS 8
 
-#define MAX(a,b)        ( (a) > (b) ? (a) : (b) )
-#define MIN(a,b)        ( (a) < (b) ? (a) : (b) )
-
 class ALSAEnumerator {
 	public:
 		QHash<QString,QString> qhInput;
@@ -48,19 +45,13 @@ class ALSAEnumerator {
 
 static ALSAEnumerator *cards = NULL;
 
-class ALSAAudioInit : public DeferInit {
-	void initialize() { cards = new ALSAEnumerator(); };
-	void destroy() { delete cards; cards = NULL; };
-};
-
-static ALSAAudioInit aai;
-
 class ALSAAudioInputRegistrar : public AudioInputRegistrar {
 	public:
 		ALSAAudioInputRegistrar();
 		virtual AudioInput *create();
 		virtual const QList<audioDevice> getDeviceChoices();
-		virtual void setDeviceChoice(const QVariant &);
+		virtual void setDeviceChoice(const QVariant &, Settings &);
+		virtual bool canEcho(const QString &);
 };
 
 
@@ -69,13 +60,46 @@ class ALSAAudioOutputRegistrar : public AudioOutputRegistrar {
 		ALSAAudioOutputRegistrar();
 		virtual AudioOutput *create();
 		virtual const QList<audioDevice> getDeviceChoices();
-		virtual void setDeviceChoice(const QVariant &);
+		virtual void setDeviceChoice(const QVariant &, Settings &);
 };
 
-static ALSAAudioInputRegistrar airALSA;
-static ALSAAudioOutputRegistrar aorALSA;
+class ALSAInit : public DeferInit {
+	protected:
+		ALSAAudioInputRegistrar *pairALSA;
+		ALSAAudioOutputRegistrar *paorALSA;
+	public:
+		void initialize();
+		void destroy();
+};
 
-ALSAAudioInputRegistrar::ALSAAudioInputRegistrar() : AudioInputRegistrar(QLatin1String("ALSA")) {
+static ALSAInit aiInit;
+
+void ALSAInit::initialize() {
+	pairALSA = NULL;
+	paorALSA = NULL;
+	cards = NULL;
+
+	int card = -1;
+	snd_card_next(&card);
+	if (card != -1) {
+		pairALSA = new ALSAAudioInputRegistrar();
+		paorALSA = new ALSAAudioOutputRegistrar();
+		cards = new ALSAEnumerator();
+	} else {
+		qWarning("ALSAInit: No cards found, not initializing");
+	}
+}
+
+void ALSAInit::destroy() {
+	if (pairALSA)
+		delete pairALSA;
+	if (paorALSA)
+		delete paorALSA;
+	if (cards)
+		delete cards;
+}
+
+ALSAAudioInputRegistrar::ALSAAudioInputRegistrar() : AudioInputRegistrar(QLatin1String("ALSA"),5) {
 }
 
 AudioInput *ALSAAudioInputRegistrar::create() {
@@ -97,16 +121,19 @@ const QList<audioDevice> ALSAAudioInputRegistrar::getDeviceChoices() {
 		QString t=QString::fromLatin1("[%1] %2").arg(dev).arg(cards->qhInput.value(dev));
 		qlReturn << audioDevice(t, dev);
 	}
-	
+
 	return qlReturn;
 }
 
-void ALSAAudioInputRegistrar::setDeviceChoice(const QVariant &choice) {
-	g.s.qsALSAInput = choice.toString();
+void ALSAAudioInputRegistrar::setDeviceChoice(const QVariant &choice, Settings &s) {
+	s.qsALSAInput = choice.toString();
 }
 
+bool ALSAAudioInputRegistrar::canEcho(const QString &) {
+	return false;
+}
 
-ALSAAudioOutputRegistrar::ALSAAudioOutputRegistrar() : AudioOutputRegistrar(QLatin1String("ALSA")) {
+ALSAAudioOutputRegistrar::ALSAAudioOutputRegistrar() : AudioOutputRegistrar(QLatin1String("ALSA"), 5) {
 }
 
 AudioOutput *ALSAAudioOutputRegistrar::create() {
@@ -132,20 +159,13 @@ const QList<audioDevice> ALSAAudioOutputRegistrar::getDeviceChoices() {
 	return qlReturn;
 }
 
-void ALSAAudioOutputRegistrar::setDeviceChoice(const QVariant &choice) {
-	g.s.qsALSAOutput = choice.toString();
+void ALSAAudioOutputRegistrar::setDeviceChoice(const QVariant &choice, Settings &s) {
+	s.qsALSAOutput = choice.toString();
 }
 
-static ConfigWidget *ALSAConfigDialogNew(Settings &st) {
-	return new ALSAConfig(st);
-}
-
-static ConfigRegistrar registrar(20, ALSAConfigDialogNew);
-
-
 ALSAEnumerator::ALSAEnumerator() {
-	qhInput.insert(QLatin1String("default"), ALSAConfig::tr("Default ALSA Card"));
-	qhOutput.insert(QLatin1String("default"), ALSAConfig::tr("Default ALSA Card"));
+	qhInput.insert(QLatin1String("default"), ALSAAudioInput::tr("Default ALSA Card"));
+	qhOutput.insert(QLatin1String("default"), ALSAAudioOutput::tr("Default ALSA Card"));
 
 	int card=-1;
 	snd_card_next(&card);
@@ -174,7 +194,6 @@ ALSAEnumerator::ALSAEnumerator() {
 				if (snd_ctl_pcm_info(ctl,info) == 0) {
 					QString fname=QString::fromLatin1(snd_pcm_info_get_name(info));
 					qhInput.insert(devname,fname);
-					qhInput.insert(QString::fromLatin1("plug:\"")+devname+QString::fromLatin1("\""), fname);
 					cap = true;
 				}
 
@@ -182,17 +201,16 @@ ALSAEnumerator::ALSAEnumerator() {
 				if (snd_ctl_pcm_info(ctl,info) == 0) {
 					QString fname=QString::fromLatin1(snd_pcm_info_get_name(info));
 					qhOutput.insert(devname,fname);
-					qhOutput.insert(QString::fromLatin1("plug:\"")+devname+QString::fromLatin1("\""), fname);
 					play = true;
 				}
 
 				snd_ctl_pcm_next_device(ctl, &device);
 			}
 			if (play) {
-				qhOutput.insert(QString::fromLatin1("plug:\"dmix:CARD=%1\"").arg(card),QLatin1String(cname));
+				qhOutput.insert(QString::fromLatin1("dmix:CARD=%1").arg(card),QLatin1String(cname));
 			}
 			if (cap) {
-				qhInput.insert(QString::fromLatin1("plug:\"dsnoop:CARD=%1\"").arg(card),QLatin1String(cname));
+				qhInput.insert(QString::fromLatin1("dsnoop:CARD=%1").arg(card),QLatin1String(cname));
 			}
 			snd_pcm_info_free(info);
 			snd_ctl_close(ctl);
@@ -201,86 +219,6 @@ ALSAEnumerator::ALSAEnumerator() {
 	}
 }
 
-ALSAConfig::ALSAConfig(Settings &st) : ConfigWidget(st) {
-	setupUi(this);
-
-	QList<QString> qlOutputDevs = cards->qhOutput.keys();
-	qSort(qlOutputDevs);
-	QList<QString> qlInputDevs = cards->qhInput.keys();
-	qSort(qlInputDevs);
-
-	bool found;
-
-	found = false;
-	foreach(QString dev, qlInputDevs) {
-		QString t=QString::fromLatin1("[%1] %2").arg(dev).arg(cards->qhInput.value(dev));
-		qcbInputDevice->addItem(t, dev);
-		if (dev == g.s.qsALSAInput) {
-			found = true;
-			qcbInputDevice->setCurrentIndex(qcbInputDevice->count() - 1);
-		}
-	}
-	if (! found) {
-		qcbInputDevice->addItem(g.s.qsALSAInput, g.s.qsALSAInput);
-		qcbInputDevice->setCurrentIndex(qcbInputDevice->count() - 1);
-	}
-
-	found = false;
-	foreach(QString dev, qlOutputDevs) {
-		QString t=QString::fromLatin1("[%1] %2").arg(dev).arg(cards->qhOutput.value(dev));
-		qcbOutputDevice->addItem(t, dev);
-		if (dev == g.s.qsALSAOutput) {
-			found = true;
-			qcbOutputDevice->setCurrentIndex(qcbOutputDevice->count() - 1);
-		}
-	}
-	if (! found) {
-		qcbOutputDevice->addItem(g.s.qsALSAOutput, g.s.qsALSAOutput);
-		qcbOutputDevice->setCurrentIndex(qcbOutputDevice->count() - 1);
-	}
-
-	qcbOutputDevice->setWhatsThis(qcbInputDevice->whatsThis());
-}
-
-QString ALSAConfig::title() const {
-	return tr("ALSA");
-}
-
-QIcon ALSAConfig::icon() const {
-	return QIcon(QLatin1String("skin:config_dsound.png"));
-}
-
-void ALSAConfig::save() const {
-	s.iDXOutputDelay = qsOutputDelay->value();
-	s.qsALSAInput = qcbInputDevice->itemData(qcbInputDevice->currentIndex()).toString();
-	s.qsALSAOutput = qcbOutputDevice->itemData(qcbOutputDevice->currentIndex()).toString();
-}
-
-void ALSAConfig::load(const Settings &r) {
-	for (int i=0;i<qcbInputDevice->count();i++) {
-		if (qcbInputDevice->itemData(i).toString() == r.qsALSAInput) {
-			loadComboBox(qcbInputDevice, i);
-			break;
-		}
-	}
-
-	for (int i=0;i<qcbOutputDevice->count();i++) {
-		if (qcbOutputDevice->itemData(i).toString() == r.qsALSAOutput) {
-			loadComboBox(qcbOutputDevice, i);
-			break;
-		}
-	}
-	loadSlider(qsOutputDelay, r.iDXOutputDelay);
-}
-
-bool ALSAConfig::expert(bool b) {
-	qcbOutput->setVisible(b);
-	return true;
-}
-
-void ALSAConfig::on_qsOutputDelay_valueChanged(int v) {
-	qlOutputDelay->setText(tr("%1ms").arg(v*20));
-}
 
 ALSAAudioInput::ALSAAudioInput() {
 	bRunning = true;
@@ -292,37 +230,47 @@ ALSAAudioInput::~ALSAAudioInput() {
 	wait();
 }
 
-#define ALSA_ERRBAIL(x) if (!bOk) {} else if ((err=(x)) != 0) bOk = false
+#define ALSA_ERRBAIL(x) if (!bOk) {} else if ((err=(x)) != 0) { bOk = false; qWarning("ALSAAudio: %s: %s", #x, snd_strerror(err));}
 
 void ALSAAudioInput::run() {
 	int readblapp;
 
-
 	QByteArray device_name = g.s.qsALSAInput.toLatin1();
 	snd_pcm_hw_params_t *hw_params = NULL;
 	snd_pcm_t *capture_handle = NULL;
-	snd_pcm_uframes_t wantPeriod = iFrameSize;
-	snd_pcm_uframes_t wantBuff = wantPeriod * 4;
 
 	unsigned int rrate = SAMPLE_RATE;
 	bool bOk = true;
 
 	int err = 0;
 
+	unsigned int iChannels = 1;
+
 	qWarning("ALSAAudioInput: Initing audiocapture %s.",device_name.data());
 
+	snd_pcm_hw_params_alloca(&hw_params);
+
 	ALSA_ERRBAIL(snd_pcm_open(&capture_handle, device_name.data(), SND_PCM_STREAM_CAPTURE, 0));
-	ALSA_ERRBAIL(snd_pcm_hw_params_malloc(&hw_params));
 	ALSA_ERRBAIL(snd_pcm_hw_params_any(capture_handle, hw_params));
 	ALSA_ERRBAIL(snd_pcm_hw_params_set_access(capture_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED));
-	ALSA_ERRBAIL(snd_pcm_hw_params_set_format(capture_handle, hw_params, SND_PCM_FORMAT_S16_LE));
+	ALSA_ERRBAIL(snd_pcm_hw_params_set_format(capture_handle, hw_params, SND_PCM_FORMAT_S16));
 	ALSA_ERRBAIL(snd_pcm_hw_params_set_rate_near(capture_handle, hw_params, &rrate, NULL));
-	ALSA_ERRBAIL(snd_pcm_hw_params_set_channels(capture_handle, hw_params, 1));
+	ALSA_ERRBAIL(snd_pcm_hw_params_set_channels_near(capture_handle, hw_params, &iChannels));
+
+	snd_pcm_uframes_t wantPeriod = (rrate * iFrameSize) / SAMPLE_RATE;
+	snd_pcm_uframes_t wantBuff = wantPeriod * 8;
+
 	ALSA_ERRBAIL(snd_pcm_hw_params_set_period_size_near(capture_handle, hw_params, &wantPeriod, NULL));
 	ALSA_ERRBAIL(snd_pcm_hw_params_set_buffer_size_near(capture_handle, hw_params, &wantBuff));
 	ALSA_ERRBAIL(snd_pcm_hw_params(capture_handle, hw_params));
 
-	qWarning("ALSAAudioInput: Actual buffer %ld samples [%ld(%d) per period]",wantBuff,wantPeriod,iFrameSize);
+	qWarning("ALSAAudioInput: Actual buffer %d hz, %d channel %ld samples [%ld per period]",rrate,iChannels,wantBuff,wantPeriod);
+
+	ALSA_ERRBAIL(snd_pcm_hw_params_current(capture_handle, hw_params));
+	ALSA_ERRBAIL(snd_pcm_hw_params_get_channels(hw_params, &iMicChannels));
+	ALSA_ERRBAIL(snd_pcm_hw_params_get_rate(hw_params, &iMicFreq, NULL));
+	eMicFormat = SampleShort;
+	initializeMixer();
 
 #ifdef ALSA_VERBOSE
 	snd_output_t *log;
@@ -334,9 +282,6 @@ void ALSAAudioInput::run() {
 	ALSA_ERRBAIL(snd_pcm_prepare(capture_handle));
 	ALSA_ERRBAIL(snd_pcm_start(capture_handle));
 
-	if (hw_params)
-		snd_pcm_hw_params_free(hw_params);
-
 	if (! bOk) {
 		if (capture_handle) {
 			snd_pcm_drain(capture_handle);
@@ -347,6 +292,8 @@ void ALSAAudioInput::run() {
 		return;
 	}
 
+	char inbuff[wantPeriod * iChannels * sizeof(float)];
+
 	while (bRunning) {
 #ifdef ALSA_VERBOSE
 		snd_pcm_status_malloc(&status);
@@ -354,7 +301,7 @@ void ALSAAudioInput::run() {
 		snd_pcm_status_dump(status, log);
 		snd_pcm_status_free(status);
 #endif
-		readblapp = snd_pcm_readi(capture_handle, psMic, iFrameSize);
+		readblapp = snd_pcm_readi(capture_handle, inbuff, wantPeriod);
 		if (readblapp == -ESTRPIPE) {
 			// suspend event - what to do?
 			qWarning("ALSAAudioInput: %s", snd_strerror(readblapp));
@@ -364,8 +311,8 @@ void ALSAAudioInput::run() {
 		} else if (readblapp < 0) {
 			err = snd_pcm_prepare(capture_handle);
 			qWarning("ALSAAudioInput: %s: %s", snd_strerror(readblapp), snd_strerror(err));
-		} else if (iFrameSize == readblapp) {
-			encodeAudioFrame();
+		} else if (wantPeriod == static_cast<unsigned int>(readblapp)) {
+			addMic(inbuff, readblapp);
 		}
 	}
 
@@ -375,67 +322,6 @@ void ALSAAudioInput::run() {
 	qWarning("ALSAAudioInput: Releasing ALSA Mic.");
 }
 
-void ALSAAudioOutput::initialize(snd_pcm_t * &pcm_handle, int period) {
-	int err = 0;
-	bool bOk = true;
-
-	if (pcm_handle)
-		return;
-
-	short zerobuff[period];
-	for (int i=0;i<period;i++)
-		zerobuff[i]=0;
-
-	unsigned int rate = SAMPLE_RATE;
-	snd_pcm_uframes_t period_size = period;
-	snd_pcm_uframes_t buffer_size = period * (g.s.iDXOutputDelay + 1);
-
-	snd_pcm_hw_params_t *hw_params = NULL;
-	snd_pcm_sw_params_t *sw_params = NULL;
-	QByteArray device_name = g.s.qsALSAOutput.toLatin1();
-
-	ALSA_ERRBAIL(snd_pcm_open(&pcm_handle, device_name.data(), SND_PCM_STREAM_PLAYBACK, 0));
-	ALSA_ERRBAIL(snd_pcm_hw_params_malloc(&hw_params));
-	ALSA_ERRBAIL(snd_pcm_sw_params_malloc(&sw_params));
-
-	ALSA_ERRBAIL(snd_pcm_hw_params_any(pcm_handle, hw_params));
-	ALSA_ERRBAIL(snd_pcm_hw_params_set_access(pcm_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED));
-	ALSA_ERRBAIL(snd_pcm_hw_params_set_format(pcm_handle, hw_params, SND_PCM_FORMAT_S16_LE));
-	ALSA_ERRBAIL(snd_pcm_hw_params_set_rate_near(pcm_handle, hw_params, &rate, NULL));
-	ALSA_ERRBAIL(snd_pcm_hw_params_set_channels(pcm_handle, hw_params, 1));
-	ALSA_ERRBAIL(snd_pcm_hw_params_set_buffer_size_near(pcm_handle, hw_params, &buffer_size));
-	ALSA_ERRBAIL(snd_pcm_hw_params_set_period_size_near(pcm_handle, hw_params, &period_size, NULL));
-
-	ALSA_ERRBAIL(snd_pcm_hw_params(pcm_handle, hw_params));
-
-	ALSA_ERRBAIL(snd_pcm_sw_params_current(pcm_handle, sw_params));
-	ALSA_ERRBAIL(snd_pcm_sw_params_set_start_threshold(pcm_handle, sw_params, 0));
-	ALSA_ERRBAIL(snd_pcm_sw_params_set_avail_min(pcm_handle, sw_params, period_size));
-
-	ALSA_ERRBAIL(snd_pcm_sw_params(pcm_handle, sw_params));
-
-	ALSA_ERRBAIL(snd_pcm_prepare(pcm_handle));
-
-	// Fill one frame
-	if (bOk && pcm_handle)
-		for (int i = 0; i < g.s.iDXOutputDelay + 1; i++)
-			snd_pcm_writei(pcm_handle, zerobuff, period);
-
-	if (sw_params)
-		snd_pcm_sw_params_free(sw_params);
-	if (hw_params)
-		snd_pcm_hw_params_free(hw_params);
-
-	if (! bOk) {
-		g.mw->msgBox(tr("Opening chosen ALSA Output failed: %1").arg(QLatin1String(snd_strerror(err))));
-		if (pcm_handle) {
-			snd_pcm_close(pcm_handle);
-			pcm_handle = NULL;
-		}
-		return;
-	}
-}
-
 ALSAAudioOutput::ALSAAudioOutput() {
 	qWarning("ALSAAudioOutput: Initialized");
 	bRunning = true;
@@ -455,18 +341,98 @@ void ALSAAudioOutput::run() {
 	struct pollfd fds[16];
 	int count;
 	bool stillRun = true;
+	int err = 0;
+	bool bOk = true;
 
-	initialize(pcm_handle, iFrameSize);
+	iMixerFreq = SAMPLE_RATE;
 
-	if (! pcm_handle)
-		return;
+	snd_pcm_hw_params_t *hw_params = NULL;
+	snd_pcm_sw_params_t *sw_params = NULL;
+	QByteArray device_name = g.s.qsALSAOutput.toLatin1();
 
-	short buffer[iFrameSize] __attribute__((aligned(16)));
+	snd_pcm_hw_params_alloca(&hw_params);
+	snd_pcm_sw_params_alloca(&sw_params);
 
-	short zerobuff[iFrameSize];
-	for (int i=0;i<iFrameSize;i++)
+	ALSA_ERRBAIL(snd_pcm_open(&pcm_handle, device_name.data(), SND_PCM_STREAM_PLAYBACK, 0));
+	ALSA_ERRBAIL(snd_pcm_hw_params_any(pcm_handle, hw_params));
+
+	if (g.s.doPositionalAudio()) {
+		iChannels = 1;
+		ALSA_ERRBAIL(snd_pcm_hw_params_get_channels_max(hw_params, &iChannels));
+		if (iChannels > 8) {
+			qWarning("ALSAAudioOutput: ALSA reports %d output channels. Clamping to 2.",iChannels);
+			iChannels = 2;
+		}
+	} else {
+		iChannels = 1;
+	}
+
+	ALSA_ERRBAIL(snd_pcm_hw_params_set_access(pcm_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED));
+	ALSA_ERRBAIL(snd_pcm_hw_params_set_format(pcm_handle, hw_params, SND_PCM_FORMAT_S16));
+	ALSA_ERRBAIL(snd_pcm_hw_params_set_rate_min(pcm_handle, hw_params, &iMixerFreq, NULL));
+	ALSA_ERRBAIL(snd_pcm_hw_params_set_channels_near(pcm_handle, hw_params, &iChannels));
+
+	unsigned int iOutputSize = (iFrameSize * iMixerFreq) / SAMPLE_RATE;
+
+	snd_pcm_uframes_t period_size = iOutputSize;
+	snd_pcm_uframes_t buffer_size = iOutputSize * (g.s.iOutputDelay + 1);
+
+	ALSA_ERRBAIL(snd_pcm_hw_params_set_period_size_near(pcm_handle, hw_params, &period_size, NULL));
+	ALSA_ERRBAIL(snd_pcm_hw_params_set_buffer_size_near(pcm_handle, hw_params, &buffer_size));
+
+	ALSA_ERRBAIL(snd_pcm_hw_params(pcm_handle, hw_params));
+	ALSA_ERRBAIL(snd_pcm_hw_params_current(pcm_handle, hw_params));
+
+	ALSA_ERRBAIL(snd_pcm_sw_params_current(pcm_handle, sw_params));
+	ALSA_ERRBAIL(snd_pcm_sw_params_set_start_threshold(pcm_handle, sw_params, 0));
+	ALSA_ERRBAIL(snd_pcm_sw_params_set_avail_min(pcm_handle, sw_params, period_size));
+
+	ALSA_ERRBAIL(snd_pcm_sw_params(pcm_handle, sw_params));
+
+	ALSA_ERRBAIL(snd_pcm_prepare(pcm_handle));
+
+	const unsigned int buffsize = period_size * iChannels;
+
+	float zerobuff[buffsize];
+	float outbuff[buffsize];
+
+	for (unsigned int i=0;i<buffsize;i++)
 		zerobuff[i]=0;
 
+	// Fill buffer
+	if (bOk && pcm_handle)
+		for (unsigned int i = 0; i < buffer_size / period_size; i++)
+			snd_pcm_writei(pcm_handle, zerobuff, period_size);
+
+	if (! bOk) {
+		g.mw->msgBox(tr("Opening chosen ALSA Output failed: %1").arg(QLatin1String(snd_strerror(err))));
+		if (pcm_handle) {
+			snd_pcm_close(pcm_handle);
+			pcm_handle = NULL;
+		}
+		return;
+	}
+
+	const unsigned int chanmasks[32] = {
+		SPEAKER_FRONT_LEFT,
+		SPEAKER_FRONT_RIGHT,
+		SPEAKER_FRONT_CENTER,
+		SPEAKER_LOW_FREQUENCY,
+		SPEAKER_BACK_LEFT,
+		SPEAKER_BACK_RIGHT,
+		SPEAKER_SIDE_LEFT,
+		SPEAKER_SIDE_RIGHT,
+		SPEAKER_BACK_CENTER
+	};
+
+	ALSA_ERRBAIL(snd_pcm_hw_params_current(pcm_handle, hw_params));
+	ALSA_ERRBAIL(snd_pcm_hw_params_get_channels(hw_params, &iChannels));
+	ALSA_ERRBAIL(snd_pcm_hw_params_get_rate(hw_params, &iMixerFreq, NULL));
+	eSampleFormat = SampleShort;
+
+	qWarning("ALSAAudioOutput: Initializing %d channel, %d hz mixer", iChannels, iMixerFreq);
+	initializeMixer(chanmasks);
+
 	count = snd_pcm_poll_descriptors_count(pcm_handle);
 	snd_pcm_poll_descriptors(pcm_handle, fds, count);
 
@@ -478,24 +444,26 @@ void ALSAAudioOutput::run() {
 		if (revents & POLLERR) {
 			snd_pcm_prepare(pcm_handle);
 		} else if (revents & POLLOUT) {
-			int avail = snd_pcm_avail_update(pcm_handle);
-			while (avail >= iFrameSize) {
-				stillRun = mixAudio(buffer);
-				int w=snd_pcm_writei(pcm_handle, buffer, iFrameSize);
+			while (snd_pcm_avail_update(pcm_handle) >= period_size) {
+				stillRun = mix(outbuff, period_size);
+				int w = 0;
+				if (stillRun)
+					w=snd_pcm_writei(pcm_handle, outbuff, period_size);
+				else
+					break;
 				if (w == -EPIPE) {
 					qWarning("ALSAAudioOutput: %s", snd_strerror(w));
 					snd_pcm_prepare(pcm_handle);
-					for (int i=0;i<g.s.iDXOutputDelay;i++)
-						snd_pcm_writei(pcm_handle, zerobuff, iFrameSize);
+					for (unsigned int i=0;i< buffer_size / period_size;i++)
+						snd_pcm_writei(pcm_handle, zerobuff, period_size);
 				}
-				avail = snd_pcm_avail_update(pcm_handle);
 			}
 
 			if (! stillRun) {
 				snd_pcm_drain(pcm_handle);
 
-				while (! mixAudio(buffer) && bRunning)
-					this->usleep(20 * 1000);
+				while (bRunning && !mix(outbuff, period_size))
+					this->msleep(20);
 
 				if (! bRunning)
 					break;
@@ -503,8 +471,10 @@ void ALSAAudioOutput::run() {
 				snd_pcm_prepare(pcm_handle);
 
 				// Fill one frame
-				for (int i = 0; i < g.s.iDXOutputDelay; i++)
-					snd_pcm_writei(pcm_handle, zerobuff, iFrameSize);
+				for (unsigned int i = 0; i < (buffer_size / period_size) - 1 ; i++)
+					snd_pcm_writei(pcm_handle, zerobuff, period_size);
+
+				snd_pcm_writei(pcm_handle, outbuff, period_size);
 			}
 		}
 	}
