@@ -44,13 +44,11 @@ class ASIOAudioInputRegistrar : public AudioInputRegistrar {
 		ASIOAudioInputRegistrar();
 		virtual AudioInput *create();
 		virtual const QList<audioDevice> getDeviceChoices();
-		virtual void setDeviceChoice(const QVariant &);
+		virtual void setDeviceChoice(const QVariant &, Settings &);
+		virtual bool canEcho(const QString &);
 
 };
 
-// Static singleton
-static ASIOAudioInputRegistrar airASIO;
-
 ASIOAudioInputRegistrar::ASIOAudioInputRegistrar() : AudioInputRegistrar(QLatin1String("ASIO")) {
 }
 
@@ -62,16 +60,82 @@ const QList<audioDevice> ASIOAudioInputRegistrar::getDeviceChoices() {
 	return qlReturn;
 }
 
-void ASIOAudioInputRegistrar::setDeviceChoice(const QVariant &) {
-	qWarning("ASIOInputRegistrar::setDeviceChoice was called");
+bool ASIOAudioInputRegistrar::canEcho(const QString &) {
+	return true;
+}
+
+void ASIOAudioInputRegistrar::setDeviceChoice(const QVariant &, Settings &) {
 }
 
 static ConfigWidget *ASIOConfigDialogNew(Settings &st) {
 	return new ASIOConfig(st);
 }
 
-static ConfigRegistrar registrar(22, ASIOConfigDialogNew);
+class ASIOInit : public DeferInit {
+		ASIOAudioInputRegistrar *airASIO;
+		ConfigRegistrar *crASIO;
+	public:
+		void initialize();
+		void destroy();
+};
 
+void ASIOInit::initialize() {
+	HKEY hkDevs;
+	HKEY hk;
+	DWORD idx = 0;
+	WCHAR keyname[255];
+	DWORD keynamelen = 255;
+	FILETIME ft;
+	HRESULT hr;
+
+	airASIO = NULL;
+	crASIO = NULL;
+
+	bool bFound = false;
+
+	// List of devices known to misbehave or be totally useless
+	QStringList blacklist;
+	blacklist << QLatin1String("{a91eaba1-cf4c-11d3-b96a-00a0c9c7b61a}"); // ASIO DirectX
+	blacklist << QLatin1String("{e3186861-3a74-11d1-aef8-0080ad153287}"); // ASIO Multimedia
+	blacklist << QLatin1String("{232685c6-6548-49d8-846d-4141a3ef7560}"); // ASIO4ALL
+
+	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, L"Software\\ASIO", 0, KEY_READ, &hkDevs) == ERROR_SUCCESS) {
+		while (RegEnumKeyEx(hkDevs, idx++, keyname, &keynamelen, NULL, NULL, NULL, &ft)  == ERROR_SUCCESS) {
+			QString name=QString::fromUtf16(reinterpret_cast<ushort *>(keyname),keynamelen);
+			if (RegOpenKeyEx(hkDevs, keyname, 0, KEY_READ, &hk) == ERROR_SUCCESS) {
+				DWORD dtype = REG_SZ;
+				WCHAR wclsid[255];
+				DWORD datasize = 255;
+				CLSID clsid;
+				if (RegQueryValueEx(hk, L"CLSID", 0, &dtype, reinterpret_cast<BYTE *>(wclsid), &datasize) == ERROR_SUCCESS) {
+					if (datasize > 76)
+						datasize = 76;
+					QString qsCls = QString::fromUtf16(reinterpret_cast<ushort *>(wclsid), datasize / 2).toLower().trimmed();
+					if (! blacklist.contains(qsCls.toLower()) && ! FAILED(hr =CLSIDFromString(wclsid, &clsid))) {
+						bFound = true;
+					}
+				}
+				RegCloseKey(hk);
+			}
+			keynamelen = 255;
+		}
+		RegCloseKey(hkDevs);
+	}
+
+	if (bFound) {
+		airASIO = new ASIOAudioInputRegistrar();
+		crASIO = new ConfigRegistrar(2002, ASIOConfigDialogNew);
+	}
+}
+
+void ASIOInit::destroy() {
+	if (airASIO)
+		delete airASIO;
+	if (crASIO)
+		delete crASIO;
+}
+
+static class ASIOInit asioinit;
 
 ASIOInput *ASIOInput::aiSelf;
 
@@ -90,6 +154,7 @@ ASIOConfig::ASIOConfig(Settings &st) : ConfigWidget(st) {
 	QStringList blacklist;
 	blacklist << QLatin1String("{a91eaba1-cf4c-11d3-b96a-00a0c9c7b61a}"); // ASIO DirectX
 	blacklist << QLatin1String("{e3186861-3a74-11d1-aef8-0080ad153287}"); // ASIO Multimedia
+	blacklist << QLatin1String("{232685c6-6548-49d8-846d-4141a3ef7560}"); // ASIO4ALL
 
 	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, L"Software\\ASIO", 0, KEY_READ, &hkDevs) == ERROR_SUCCESS) {
 		while (RegEnumKeyEx(hkDevs, idx++, keyname, &keynamelen, NULL, NULL, NULL, &ft)  == ERROR_SUCCESS) {
@@ -100,7 +165,9 @@ ASIOConfig::ASIOConfig(Settings &st) : ConfigWidget(st) {
 				DWORD datasize = 255;
 				CLSID clsid;
 				if (RegQueryValueEx(hk, L"CLSID", 0, &dtype, reinterpret_cast<BYTE *>(wclsid), &datasize) == ERROR_SUCCESS) {
-					QString qsCls = QString::fromUtf16(reinterpret_cast<ushort *>(wclsid), datasize / 2);
+					if (datasize > 76)
+						datasize = 76;
+					QString qsCls = QString::fromUtf16(reinterpret_cast<ushort *>(wclsid), datasize / 2).toLower().trimmed();
 					if (! blacklist.contains(qsCls) && ! FAILED(hr =CLSIDFromString(wclsid, &clsid))) {
 						ASIODev ad(name, qsCls);
 						qlDevs << ad;
@@ -127,44 +194,41 @@ ASIOConfig::ASIOConfig(Settings &st) : ConfigWidget(st) {
 	}
 }
 
-#include "iasiothiscallresolver.h"
-
 void ASIOConfig::on_qpbQuery_clicked() {
 	QString qsCls = qcbDevice->itemData(qcbDevice->currentIndex()).toString();
 	CLSID clsid;
-	IASIO *iorigasio;
+	IASIO *iasio;
 
 	bool ok = false;
 
 	clearQuery();
 
 	CLSIDFromString(const_cast<wchar_t *>(reinterpret_cast<const wchar_t *>(qsCls.utf16())), &clsid);
-	if (CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER, clsid, reinterpret_cast<void **>(&iorigasio)) == S_OK) {
+	if (CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER, clsid, reinterpret_cast<void **>(&iasio)) == S_OK) {
 		SleepEx(10, false);
-		IASIOThiscallResolver iasio(iorigasio);
-		if (iasio.init(winId())) {
+		if (iasio->init(winId())) {
 			SleepEx(10, false);
 			char buff[512];
 			memset(buff, 0, 512);
 
-			iasio.getDriverName(buff);
+			iasio->getDriverName(buff);
 			SleepEx(10, false);
 
-			long ver = iasio.getDriverVersion();
+			long ver = iasio->getDriverVersion();
 			SleepEx(10, false);
 
 			ASIOSampleRate srate;
-			iasio.getSampleRate(&srate);
+			iasio->getSampleRate(&srate);
 			SleepEx(10, false);
 			if (fabs(srate-48000.0) > 1.0) {
-				iasio.setSampleRate(48000.0);
+				iasio->setSampleRate(48000.0);
 				SleepEx(10, false);
-				iasio.getSampleRate(&srate);
+				iasio->getSampleRate(&srate);
 				SleepEx(10, false);
 			}
 
 			long minSize, maxSize, prefSize, granSize;
-			iasio.getBufferSize(&minSize, &maxSize, &prefSize, &granSize);
+			iasio->getBufferSize(&minSize, &maxSize, &prefSize, &granSize);
 			SleepEx(10, false);
 
 			// We need sample sizes in ms for 2 bytes at 48khz
@@ -211,7 +275,7 @@ void ASIOConfig::on_qpbQuery_clicked() {
 
 			if (bOk) {
 				long ichannels, ochannels;
-				iasio.getChannels(&ichannels, &ochannels);
+				iasio->getChannels(&ichannels, &ochannels);
 				SleepEx(10, false);
 				long cnum;
 
@@ -220,7 +284,7 @@ void ASIOConfig::on_qpbQuery_clicked() {
 					ASIOChannelInfo aci;
 					aci.channel = cnum;
 					aci.isInput = true;
-					iasio.getChannelInfo(&aci);
+					iasio->getChannelInfo(&aci);
 					SleepEx(10, false);
 					switch (aci.type) {
 						case ASIOSTInt32LSB:
@@ -246,11 +310,11 @@ void ASIOConfig::on_qpbQuery_clicked() {
 		if (! ok) {
 			SleepEx(10, false);
 			char err[255];
-			iasio.getErrorMessage(err);
+			iasio->getErrorMessage(err);
 			SleepEx(10, false);
 			QMessageBox::critical(this, tr("Mumble"), tr("ASIO Initialization failed: %1").arg(QLatin1String(err)), QMessageBox::Ok, QMessageBox::NoButton);
 		}
-		iasio.Release();
+		iasio->Release();
 	} else {
 		QMessageBox::critical(this, tr("Mumble"), tr("Failed to instantiate ASIO driver"), QMessageBox::Ok, QMessageBox::NoButton);
 	}
@@ -259,26 +323,25 @@ void ASIOConfig::on_qpbQuery_clicked() {
 void ASIOConfig::on_qpbConfig_clicked() {
 	QString qsCls = qcbDevice->itemData(qcbDevice->currentIndex()).toString();
 	CLSID clsid;
-	IASIO *iorigasio;
+	IASIO *iasio;
 
 	bool ok = false;
 
 	CLSIDFromString(const_cast<wchar_t *>(reinterpret_cast<const wchar_t *>(qsCls.utf16())), &clsid);
-	if (CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER, clsid, reinterpret_cast<void **>(&iorigasio)) == S_OK) {
+	if (CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER, clsid, reinterpret_cast<void **>(&iasio)) == S_OK) {
 		SleepEx(10, false);
-		IASIOThiscallResolver iasio(iorigasio);
-		if (iasio.init(winId())) {
+		if (iasio->init(winId())) {
 			SleepEx(10, false);
-			ok = (iasio.controlPanel() == ASE_OK);
+			ok = (iasio->controlPanel() == ASE_OK);
 			SleepEx(10, false);
 		} else {
 			SleepEx(10, false);
 			char err[255];
-			iasio.getErrorMessage(err);
+			iasio->getErrorMessage(err);
 			SleepEx(10, false);
 			QMessageBox::critical(this, tr("Mumble"), tr("ASIO Initialization failed: %1").arg(QLatin1String(err)), QMessageBox::Ok, QMessageBox::NoButton);
 		}
-		iasio.Release();
+		iasio->Release();
 	} else {
 		QMessageBox::critical(this, tr("Mumble"), tr("Failed to instantiate ASIO driver"), QMessageBox::Ok, QMessageBox::NoButton);
 	}
@@ -384,24 +447,22 @@ void ASIOConfig::clearQuery() {
 ASIOInput::ASIOInput() {
 	QString qsCls = g.s.qsASIOclass;
 	CLSID clsid;
-	IASIO *iorigasio;
 
-	iorigasio = false;
 	iasio = NULL;
 	abiInfo = NULL;
 	aciInfo = NULL;
 
 	int i, idx;
 
-	bHasSpeaker = true;
+	iEchoChannels = 1;
 
 	// Allocate buffers
 	pdInputBuffer = new float[960];
 	pdOutputBuffer = new float[iFrameSize];
 
 	int err = 0;
-	srsResampleMic = speex_resampler_init(1, 48000, 16000, 8, &err);
-	srsResampleSpeaker = speex_resampler_init(1, 48000, 16000, 8, &err);
+	srsResampleMic = speex_resampler_init(1, 48000, SAMPLE_RATE, 3, &err);
+	srsResampleSpeaker = speex_resampler_init(1, 48000, SAMPLE_RATE, 3, &err);
 
 	// Sanity check things first.
 
@@ -415,8 +476,7 @@ ASIOInput::ASIOInput() {
 	}
 
 	CLSIDFromString(const_cast<wchar_t *>(reinterpret_cast<const wchar_t *>(qsCls.utf16())), &clsid);
-	if (CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER, clsid, reinterpret_cast<void **>(&iaOriginal)) == S_OK) {
-		iasio = new IASIOThiscallResolver(iaOriginal);
+	if (CoCreateInstance(clsid, NULL, CLSCTX_INPROC_SERVER, clsid, reinterpret_cast<void **>(&iasio)) == S_OK) {
 		if (iasio->init(NULL)) {
 			ASIOSampleRate srate = 0.0;
 			iasio->getSampleRate(&srate);
@@ -473,11 +533,9 @@ ASIOInput::ASIOInput() {
 	}
 
 	if (iasio) {
-		iaOriginal->Release();
-		delete iasio;
+		iasio->Release();
 
 		iasio = NULL;
-		iaOriginal = NULL;
 	}
 
 	QMessageBox::critical(NULL, tr("Mumble"), tr("Opening selected ASIO device failed. No input will be done."),
@@ -491,8 +549,7 @@ ASIOInput::~ASIOInput() {
 	if (iasio) {
 		iasio->stop();
 		iasio->disposeBuffers();
-		iaOriginal->Release();
-		delete iasio;
+		iasio->Release();
 		iasio = NULL;
 	}
 	if (abiInfo) {
@@ -540,7 +597,6 @@ ASIOInput::addBuffer(ASIOSampleType sampType, void *src, float *dst) {
 }
 
 void
-__attribute__((force_align_arg_pointer))
 ASIOInput::bufferReady(long buffindex) {
 	int c, i;
 
@@ -549,7 +605,7 @@ ASIOInput::bufferReady(long buffindex) {
 	for (c=0;c<iNumMic;c++)
 		addBuffer(aciInfo[c].type, abiInfo[c].buffers[buffindex], pdInputBuffer);
 
-	double mul = 1.0 / (iNumMic);
+	float mul = 1.0 / (iNumMic);
 
 	spx_uint32_t inlen, outlen;
 
