@@ -30,19 +30,24 @@
 
 #include "AudioWizard.h"
 #include "AudioInput.h"
-#include "AudioOutput.h"
 #include "Global.h"
 #include "Settings.h"
 
-AudioWizard::AudioWizard() {
+AudioWizard::AudioWizard(QWidget *p) : QWizard(p) {
 	bInit = true;
 
-	addPage(introPage());
-	addPage(devicePage());
-	addPage(deviceTuningPage());
-	addPage(volumePage());
-	addPage(triggerPage());
-	addPage(donePage());
+	setOption(QWizard::NoCancelButton, false);
+#ifndef Q_OS_MAC
+	resize(700, 500);
+#endif
+
+	addPage(qwpIntro = introPage());
+	addPage(qwpDevice = devicePage());
+	addPage(qwpDeviceTuning = deviceTuningPage());
+	addPage(qwpVolume = volumePage());
+	addPage(qwpTrigger = triggerPage());
+	addPage(qwpPositional = positionalPage());
+	addPage(qwpDone = donePage());
 	setWindowTitle(tr("Audio Tuning Wizard"));
 
 	sOldSettings = g.s;
@@ -68,11 +73,26 @@ AudioWizard::AudioWizard() {
 
 	QMetaObject::connectSlotsByName(this);
 	bInit = false;
-	resize(700, 500);
+
 	ticker->setSingleShot(false);
 	ticker->start(20);
 }
 
+bool AudioWizard::eventFilter(QObject *obj, QEvent *evt) {
+	if ((evt->type() == QEvent::MouseButtonPress) ||
+	        (evt->type() == QEvent::MouseMove)) {
+		QMouseEvent *qme = dynamic_cast<QMouseEvent *>(evt);
+		if (qme) {
+			if (qme->buttons() & Qt::LeftButton) {
+				QPointF qpf = qgvView->mapToScene(qme->pos());
+				fX = qpf.x();
+				fY = qpf.y();
+			}
+		}
+	}
+	return QWizard::eventFilter(obj, evt);
+}
+
 QWizardPage *AudioWizard::introPage() {
 	QWizardPage *qwpage = new QWizardPage;
 	qwpage->setTitle(tr("Introduction"));
@@ -122,13 +142,13 @@ QWizardPage *AudioWizard::devicePage() {
 	grid->addWidget(l, 0, 0, 1, 2);
 
 	qcbInput = new QComboBox();
-	foreach(AudioInputRegistrar *air, *AudioInputRegistrar::qmNew) {
-		qcbInput->addItem(air->name);
-		if (air->name == AudioInputRegistrar::current)
-			qcbInput->setCurrentIndex(qcbInput->count() - 1);
-		QList<audioDevice> ql= air->getDeviceChoices();
-		if (ql.count() > 0)
-			qhOldInputDevice.insert(air->name, ql.at(0).second);
+	if (AudioInputRegistrar::qmNew) {
+		foreach(AudioInputRegistrar *air, *AudioInputRegistrar::qmNew) {
+			qcbInput->addItem(air->name);
+			if (air->name == AudioInputRegistrar::current)
+				qcbInput->setCurrentIndex(qcbInput->count() - 1);
+			QList<audioDevice> ql= air->getDeviceChoices();
+		}
 	}
 
 	l = new QLabel(tr("System"));
@@ -157,6 +177,13 @@ QWizardPage *AudioWizard::devicePage() {
 	grid->addWidget(l, 2, 0);
 	grid->addWidget(qcbInputDevice, 2, 1);
 
+	qcbEcho = new QCheckBox(tr("Use echo cancellation"));
+	qcbEcho->setToolTip(tr("Cancel echo from headset or speakers."));
+	qcbEcho->setWhatsThis(tr("This enables echo cancellation of outgoing audio, which helps both on speakers and on headsets."));
+	qcbEcho->setObjectName(QLatin1String("Echo"));
+	qcbEcho->setChecked(g.s.bEcho);
+	grid->addWidget(qcbEcho, 3, 1);
+
 	qgbInput->setLayout(grid);
 
 
@@ -169,13 +196,13 @@ QWizardPage *AudioWizard::devicePage() {
 	grid->addWidget(l, 0, 0, 1, 2);
 
 	qcbOutput = new QComboBox();
-	foreach(AudioOutputRegistrar *aor, *AudioOutputRegistrar::qmNew) {
-		qcbOutput->addItem(aor->name);
-		if (aor->name == AudioOutputRegistrar::current)
-			qcbOutput->setCurrentIndex(qcbOutput->count() - 1);
-		QList<audioDevice> ql= aor->getDeviceChoices();
-		if (ql.count() > 0)
-			qhOldOutputDevice.insert(aor->name, ql.at(0).second);
+	if (AudioOutputRegistrar::qmNew) {
+		foreach(AudioOutputRegistrar *aor, *AudioOutputRegistrar::qmNew) {
+			qcbOutput->addItem(aor->name);
+			if (aor->name == AudioOutputRegistrar::current)
+				qcbOutput->setCurrentIndex(qcbOutput->count() - 1);
+			QList<audioDevice> ql= aor->getDeviceChoices();
+		}
 	}
 
 	l = new QLabel(tr("System"));
@@ -204,6 +231,13 @@ QWizardPage *AudioWizard::devicePage() {
 	grid->addWidget(l, 2, 0);
 	grid->addWidget(qcbOutputDevice, 2, 1);
 
+	qcbPositional = new QCheckBox(tr("Enable positional audio"));
+	qcbPositional->setToolTip(tr("Allows positioning of sound."));
+	qcbPositional->setWhatsThis(tr("This allows Mumble to use positional audio to place voices."));
+	qcbPositional->setObjectName(QLatin1String("Positional"));
+	qcbPositional->setChecked(g.s.bPositionalAudio);
+	grid->addWidget(qcbPositional, 3, 1);
+
 	qgbOutput->setLayout(grid);
 
 	QVBoxLayout *v=new QVBoxLayout;
@@ -218,6 +252,59 @@ QWizardPage *AudioWizard::devicePage() {
 	return qwpage;
 }
 
+QWizardPage *AudioWizard::positionalPage() {
+	QWizardPage *qwpage = new QWizardPage;
+	qwpage->setTitle(tr("Positional Audio"));
+	qwpage->setSubTitle(tr("Adjusting attenuation of positional audio."));
+
+	QVBoxLayout *v = new QVBoxLayout();
+	QLabel *l;
+
+	l = new QLabel(tr("Mumble supports positional audio for some games, and will position the voice of other players relative to their position in game. Depending on their position, the "
+	                  "volume of the voice will be changed between the speakers to simulate the direction and distance the other player is at. Such positioning depends on your speaker "
+	                  "configuration being correct in your operating system, so a test is done here."));
+	l->setWordWrap(true);
+	v->addWidget(l);
+	l = new QLabel(tr("The graph below shows the position of <font color=\"red\">you</font>, the <font color=\"yellow\">speakers</font> and a <font color=\"green\">moving sound source</font> as if seen from above. You should hear the audio move between the channels."));
+	l->setWordWrap(true);
+	v->addWidget(l);
+
+	QHBoxLayout *h = new QHBoxLayout();
+
+	QVBoxLayout *subv = new QVBoxLayout();
+
+	qcbHeadphone = new QCheckBox(tr("Use headphones"));
+	qcbHeadphone->setToolTip(tr("Use headphones instead of speakers."));
+	qcbHeadphone->setWhatsThis(tr("This ignores the OS speaker configuration and configures the positioning for headphones instead."));
+	qcbHeadphone->setObjectName(QLatin1String("Headphone"));
+	qcbHeadphone->setChecked(g.s.bPositionalHeadphone);
+	subv->addWidget(qcbHeadphone);
+
+
+	fAngle = 0.0f;
+	fX = fY = 0.0f;
+	qgsScene = NULL;
+	qgiSource = NULL;
+	asSource = NULL;
+	qgvView = new QGraphicsView();
+	qgvView->scale(1.0f, -1.0f);
+	subv->addWidget(qgvView);
+	qgvView->viewport()->installEventFilter(this);
+
+	h->addLayout(subv);
+
+	subv = new QVBoxLayout();
+
+	h->addLayout(subv);
+
+	v->addLayout(h);
+
+	qwpage->setLayout(v);
+
+
+	return qwpage;
+}
+
 QWizardPage *AudioWizard::volumePage() {
 	QWizardPage *qwpage = new QWizardPage;
 	qwpage->setTitle(tr("Volume tuning"));
@@ -373,7 +460,7 @@ QWizardPage *AudioWizard::deviceTuningPage() {
 
 	qsOutputDelay->setSingleStep(1);
 	qsOutputDelay->setPageStep(2);
-	qsOutputDelay->setValue(g.s.iDXOutputDelay);
+	qsOutputDelay->setValue(g.s.iOutputDelay);
 	qsOutputDelay->setObjectName(QLatin1String("OutputDelay"));
 
 	qlOutputDelay=new QLabel();
@@ -387,8 +474,8 @@ QWizardPage *AudioWizard::deviceTuningPage() {
 	grid->addWidget(qlOutputDelay, 2, 2);
 
 	l = new QLabel(tr("If you position the microphone so it can pick up the speakers or headset, Mumble will measure the total audio path "
-			  "delay in your system; this means the delay from a sample is placed in an outbound buffer until it's found in "
-			  "a matching incoming buffer."));
+	                  "delay in your system; this means the delay from a sample is placed in an outbound buffer until it's found in "
+	                  "a matching incoming buffer."));
 	l->setWordWrap(true);
 	grid->addWidget(l, 3, 0, 1, 2);
 
@@ -402,6 +489,9 @@ QWizardPage *AudioWizard::deviceTuningPage() {
 void AudioWizard::on_Input_activated(int) {
 	qcbInputDevice->clear();
 
+	if (! AudioInputRegistrar::qmNew)
+		return;
+
 	AudioInputRegistrar *air = AudioInputRegistrar::qmNew->value(qcbInput->currentText());
 	QList<audioDevice> ql = air->getDeviceChoices();
 
@@ -418,6 +508,9 @@ void AudioWizard::on_InputDevice_activated(int) {
 	if (bInit)
 		return;
 
+	if (! AudioInputRegistrar::qmNew)
+		return;
+
 	boost::weak_ptr<AudioInput> wai(g.ai);
 	g.ai.reset();
 
@@ -427,9 +520,11 @@ void AudioWizard::on_InputDevice_activated(int) {
 	AudioInputRegistrar *air = AudioInputRegistrar::qmNew->value(qcbInput->currentText());
 	int idx = qcbInputDevice->currentIndex();
 	if (idx > -1) {
-		air->setDeviceChoice(qcbInputDevice->itemData(idx));
+		air->setDeviceChoice(qcbInputDevice->itemData(idx), g.s);
 	}
 
+	qcbEcho->setEnabled(air->canEcho(qcbOutput->currentText()));
+
 	g.ai = AudioInputPtr(air->create());
 	g.ai->start(QThread::HighestPriority);
 }
@@ -437,6 +532,9 @@ void AudioWizard::on_InputDevice_activated(int) {
 void AudioWizard::on_Output_activated(int) {
 	qcbOutputDevice->clear();
 
+	if (! AudioOutputRegistrar::qmNew)
+		return;
+
 	AudioOutputRegistrar *aor = AudioOutputRegistrar::qmNew->value(qcbOutput->currentText());
 	QList<audioDevice> ql = aor->getDeviceChoices();
 
@@ -453,6 +551,9 @@ void AudioWizard::on_OutputDevice_activated(int) {
 	if (bInit)
 		return;
 
+	if (! AudioOutputRegistrar::qmNew)
+		return;
+
 	boost::weak_ptr<AudioOutput> wai(g.ao);
 	g.ao.reset();
 
@@ -462,19 +563,24 @@ void AudioWizard::on_OutputDevice_activated(int) {
 	AudioOutputRegistrar *aor = AudioOutputRegistrar::qmNew->value(qcbOutput->currentText());
 	int idx = qcbOutputDevice->currentIndex();
 	if (idx > -1) {
-		aor->setDeviceChoice(qcbOutputDevice->itemData(idx));
+		aor->setDeviceChoice(qcbOutputDevice->itemData(idx), g.s);
 	}
 
+	AudioInputRegistrar *air = AudioInputRegistrar::qmNew->value(qcbInput->currentText());
+	qcbEcho->setEnabled(air->canEcho(qcbOutput->currentText()));
+
 	g.ao = AudioOutputPtr(aor->create());
 	g.ao->start(QThread::HighPriority);
 }
 
 void AudioWizard::on_OutputDelay_valueChanged(int v) {
 	qlOutputDelay->setText(tr("%1ms").arg(v*20));
-	g.s.iDXOutputDelay = v;
+	g.s.iOutputDelay = v;
 	on_OutputDevice_activated(0);
-	if (! bInit)
-		playChord();
+	if (! bInit) {
+		AudioOutputPtr ao = g.ao;
+		ao->playSine(0.0f, 0.0f, 0xfffffff, 0.5f);
+	}
 }
 
 void AudioWizard::on_MaxAmp_valueChanged(int v) {
@@ -483,28 +589,49 @@ void AudioWizard::on_MaxAmp_valueChanged(int v) {
 
 void AudioWizard::showPage(int v) {
 	AudioOutputPtr ao = g.ao;
-	ao->wipe();
+	if (ao)
+		ao->wipe();
+	asSource = NULL;
+
+	g.s.bMute = false;
+	g.bPosTest = false;
 
 	if (v == 2) {
 		g.s.bMute = true;
+		ao->playSine(0.0f, 0.0f, 0xfffffff, 0.5f);
+	} else if (v == 5) {
+		fX = fY = 0.0f;
+		g.s.bMute = true;
+		g.bPosTest = true;
+		if (qgsScene) {
+			delete qgsScene;
+			qgiSource = NULL;
+			qgsScene = NULL;
+		}
 		playChord();
-	} else {
-		g.s.bMute = false;
 	}
 }
 
+int AudioWizard::nextId() const {
+	int next = QWizard::nextId();
+	if (currentPage() == qwpTrigger && ! g.s.bPositionalAudio)
+		next++;
+	return next;
+}
+
 void AudioWizard::playChord() {
 	AudioOutputPtr ao = g.ao;
-	ao->playSine(261.63, 0.0, 0xfffffff, 0.0);
-	ao->playSine(329.63, 0.0, 0xfffffff, 0.0);
-	ao->playSine(392.00, 0.0, 0xfffffff, 0.0);
-	ao->playSine(0.0, 0.0, 0xfffffff, 0.5);
+	if (! ao || asSource)
+		return;
+	asSource = ao->playSine(100.0f, 0.0001f, 0xfffffff, 0.5f);
 }
 
 void AudioWizard::restartAudio() {
 	boost::weak_ptr<AudioInput> wai(g.ai);
 	boost::weak_ptr<AudioOutput> wao(g.ao);
 
+	asSource = NULL;
+
 	g.ai.reset();
 	g.ao.reset();
 
@@ -516,26 +643,25 @@ void AudioWizard::restartAudio() {
 	g.s.qsAudioOutput = qcbOutput->currentText();
 
 	g.ai = AudioInputRegistrar::newFromChoice(g.s.qsAudioInput);
-	g.ai->start(QThread::HighestPriority);
+	if (g.ai)
+		g.ai->start(QThread::HighestPriority);
 	g.ao = AudioOutputRegistrar::newFromChoice(g.s.qsAudioOutput);
-	g.ao->start(QThread::HighPriority);
+	if (g.ao)
+		g.ao->start(QThread::HighPriority);
+
+	if (qgsScene) {
+		delete qgsScene;
+		qgiSource = NULL;
+		qgsScene = NULL;
+	}
+
+	if (currentPage() == qwpPositional)
+		playChord();
 }
 
 void AudioWizard::reject() {
 	g.s = sOldSettings;
 
-	foreach(AudioInputRegistrar *air, *AudioInputRegistrar::qmNew) {
-		const QString &name = air->name;
-		if (qhOldInputDevice.contains(name))
-			air->setDeviceChoice(qhOldInputDevice.value(name));
-	}
-
-	foreach(AudioOutputRegistrar *aor, *AudioOutputRegistrar::qmNew) {
-		const QString &name = aor->name;
-		if (qhOldOutputDevice.contains(name))
-			aor->setDeviceChoice(qhOldOutputDevice.value(name));
-	}
-
 	g.s.lmLoopMode = Settings::None;
 	g.bEchoTest = false;
 	restartAudio();
@@ -545,18 +671,30 @@ void AudioWizard::reject() {
 
 void AudioWizard::accept() {
 	g.s.atTransmit = sOldSettings.atTransmit;
+	g.s.bMute = sOldSettings.bMute;
+	g.s.bDeaf = sOldSettings.bDeaf;
 	g.s.lmLoopMode = Settings::None;
 	g.bEchoTest = false;
+	g.bPosTest = false;
 	restartAudio();
 	QWizard::accept();
 }
 
+bool AudioWizard::validateCurrentPage() {
+	if (currentId() == 1) {
+		if ((qcbInput->currentIndex() < 0) || (qcbOutput->currentIndex() < 0))
+			return false;
+	}
+	return true;
+}
+
 void AudioWizard::on_Ticker_timeout() {
 	AudioInputPtr ai = g.ai;
-	if (! ai)
+	AudioOutputPtr ao = g.ao;
+	if (! ai || ! ao)
 		return;
 
-	int iPeak = static_cast<int>(32767 * pow(10.0, (ai->dPeakMic / 20.0)));
+	int iPeak = static_cast<int>(32767.f * powf(10.0f, (ai->dPeakMic / 20.0f)));
 
 	if (iTicks++ >= 50) {
 		iMaxPeak = 0;
@@ -588,21 +726,55 @@ void AudioWizard::on_Ticker_timeout() {
 	else
 		txt=tr("Audio path cannot be determined. Input not recognized.");
 	qlAudioPath->setText(txt);
+
+	if (! qgsScene) {
+		unsigned int nspeaker = 0;
+		const float *spos = ao->getSpeakerPos(nspeaker);
+		if ((nspeaker > 0) && spos) {
+			qgsScene = new QGraphicsScene(QRectF(-4.0f, -4.0f, 8.0f, 8.0f), this);
+			qgsScene->addEllipse(QRectF(-0.12f, -0.12f, 0.24f, 0.24f), QPen(Qt::black), QBrush(Qt::darkRed));
+			for (unsigned int i=0;i<nspeaker;++i) {
+				if ((spos[3*i] != 0.0f) || (spos[3*i+1] != 0.0f) || (spos[3*i+2] != 0.0f))
+					qgsScene->addEllipse(QRectF(spos[3*i] - 0.1f, spos[3*i+2] - 0.1f, 0.2f, 0.2f), QPen(Qt::black), QBrush(Qt::yellow));
+			}
+			qgiSource = qgsScene->addEllipse(QRectF(-.15f, -.15f, 0.3f, 0.3f), QPen(Qt::black), QBrush(Qt::green));
+			qgvView->setScene(qgsScene);
+			qgvView->fitInView(-4.0f, -4.0f, 8.0f, 8.0f, Qt::KeepAspectRatio);
+
+			qWarning("It's at %f %f", qgiSource->x(), qgiSource->y());
+		}
+	} else if (currentPage() == qwpPositional) {
+		float xp, yp;
+		if ((fX == 0.0f) && (fY == 0.0f)) {
+			fAngle += 0.05;
+
+			xp = sin(fAngle) * 2.0f;
+			yp = cos(fAngle) * 2.0f;
+		} else {
+			xp = fX;
+			yp = fY;
+		}
+
+		qgiSource->setPos(xp, yp);
+		asSource->fPos[0] = xp;
+		asSource->fPos[1] = 0;
+		asSource->fPos[2] = yp;
+	}
 }
 
 void AudioWizard::on_VADmin_valueChanged(int v) {
-	g.s.fVADmin = v / 32767.0;
+	g.s.fVADmin = v / 32767.0f;
 }
 
 void AudioWizard::on_VADmax_valueChanged(int v) {
-	g.s.fVADmax = v / 32767.0;
+	g.s.fVADmax = v / 32767.0f;
 }
 
 void AudioWizard::on_Holdtime_valueChanged(int v) {
 	g.s.iVoiceHold = v;
 
-	double val = v * 20;
-	val = val / 1000.0;
+	float val = v * 20;
+	val = val / 1000.0f;
 	qlHoldtime->setText(tr("%1 s").arg(val, 0, 'f', 2));
 }
 
@@ -615,3 +787,18 @@ void AudioWizard::on_Amplitude_clicked(bool on) {
 	if (on)
 		g.s.vsVAD = Settings::Amplitude;
 }
+
+void AudioWizard::on_Echo_clicked(bool on) {
+	g.s.bEcho = on;
+	restartAudio();
+}
+
+void AudioWizard::on_Headphone_clicked(bool on) {
+	g.s.bPositionalHeadphone = on;
+	restartAudio();
+}
+
+void AudioWizard::on_Positional_clicked(bool on) {
+	g.s.bPositionalAudio = on;
+	restartAudio();
+}
