@@ -33,7 +33,6 @@
 #include "Plugins.h"
 #include "Player.h"
 #include "Global.h"
-#include "DXConfigDialog.h"
 
 #undef FAILED
 #define FAILED(Status) (static_cast<HRESULT>(Status)<0)
@@ -51,7 +50,7 @@ class DXAudioOutputRegistrar : public AudioOutputRegistrar {
 		DXAudioOutputRegistrar();
 		virtual AudioOutput *create();
 		virtual const QList<audioDevice> getDeviceChoices();
-		virtual void setDeviceChoice(const QVariant &);
+		virtual void setDeviceChoice(const QVariant &, Settings &);
 
 };
 
@@ -80,7 +79,7 @@ static BOOL CALLBACK DSEnumProc(LPGUID lpGUID, const WCHAR* lpszDesc,
 const QList<audioDevice> DXAudioOutputRegistrar::getDeviceChoices() {
 	QList<dsDevice> qlOutput;
 
-	qlOutput << dsDevice(DXConfigDialog::tr("Default DirectSound Voice Output"), DSDEVID_DefaultVoicePlayback);
+	qlOutput << dsDevice(DXAudioOutput::tr("Default DirectSound Voice Output"), DSDEVID_DefaultVoicePlayback);
 	DirectSoundEnumerate(DSEnumProc, reinterpret_cast<void *>(&qlOutput));
 
 	QList<audioDevice> qlReturn;
@@ -106,8 +105,8 @@ const QList<audioDevice> DXAudioOutputRegistrar::getDeviceChoices() {
 	return qlReturn;
 }
 
-void DXAudioOutputRegistrar::setDeviceChoice(const QVariant &choice) {
-	g.s.qbaDXOutput = choice.toByteArray();
+void DXAudioOutputRegistrar::setDeviceChoice(const QVariant &choice, Settings &s) {
+	s.qbaDXOutput = choice.toByteArray();
 }
 
 
@@ -120,12 +119,12 @@ DXAudioOutputPlayer::DXAudioOutputPlayer(DXAudioOutput *ao, AudioOutputPlayer *a
 	pDSNotify = NULL;
 	pDS3dBuffer = NULL;
 
-	iByteSize = aop->iFrameSize * 2;
+	iByteSize = 320 * sizeof(short);
 
 	hNotificationEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
 }
 
-void DXAudioOutputPlayer::setupAudioDevice() {
+bool DXAudioOutputPlayer::setupAudioDevice() {
 	DSBUFFERDESC dsbd;
 	WAVEFORMATEX wfx;
 	HRESULT hr;
@@ -144,58 +143,54 @@ void DXAudioOutputPlayer::setupAudioDevice() {
 	dsbd.dwFlags	 |= DSBCAPS_CTRLPOSITIONNOTIFY;
 	if (dxAudio->p3DListener)
 		dsbd.dwFlags	 |= DSBCAPS_CTRL3D;
-	dsbd.dwBufferBytes = aop->iFrameSize * 2 * NBLOCKS;
+	dsbd.dwBufferBytes = 320 * sizeof(short) * NBLOCKS;
 	dsbd.lpwfxFormat     = &wfx;
 	if (dxAudio->p3DListener) {
-		switch (g.s.a3dModel) {
-			case Settings::None:
-			case Settings::Panning:
-				dsbd.guid3DAlgorithm = DS3DALG_NO_VIRTUALIZATION;
-				break;
-			case Settings::Light:
-				dsbd.guid3DAlgorithm = DS3DALG_HRTF_LIGHT;
-				break;
-			case Settings::Full:
-				dsbd.guid3DAlgorithm = DS3DALG_HRTF_FULL;
-				break;
-		}
+		dsbd.guid3DAlgorithm = DS3DALG_NO_VIRTUALIZATION;
 	}
 
 	// Create the DirectSound buffer
-	if (FAILED(hr = dxAudio->pDS->CreateSoundBuffer(&dsbd, &pDSBOutput, NULL)))
-		qFatal("DXAudioOutputPlayer: CreateSoundBuffer (Secondary): 0x%08lx", hr);
+	if (FAILED(hr = dxAudio->pDS->CreateSoundBuffer(&dsbd, &pDSBOutput, NULL))) {
+		qWarning("DXAudioOutputPlayer: CreateSoundBuffer (Secondary): 0x%08lx", hr);
+		return false;
+	}
 
 	DSBPOSITIONNOTIFY    aPosNotify[NBLOCKS];
 
 	for (int i=0;i<NBLOCKS;i++) {
-		aPosNotify[i].dwOffset = aop->iFrameSize * 2 * i;
+		aPosNotify[i].dwOffset = 320 * sizeof(short) * i;
 		aPosNotify[i].hEventNotify = hNotificationEvent;
 	}
 
-	if (FAILED(hr = pDSBOutput->SetVolume(lround(log10(g.s.fVolume)*5000.0))))
-		qFatal("DXAudioOutputPlayer: Failed to set volume");
-
-	if (FAILED(hr = pDSBOutput->QueryInterface(IID_IDirectSoundNotify, reinterpret_cast<void **>(&pDSNotify))))
-		qFatal("DXAudioOutputPlayer: QueryInterface (Notify)");
-
-	if (FAILED(hr = pDSNotify->SetNotificationPositions(NBLOCKS, aPosNotify)))
-		qFatal("DXAudioOutputPlayer: SetNotificationPositions");
-
-	if (dxAudio->p3DListener) {
-		if (FAILED(pDSBOutput->QueryInterface(IID_IDirectSound3DBuffer8, reinterpret_cast<void **>(&pDS3dBuffer))))
-			qFatal("DXAudioOutputPlayer: QueryInterface (DirectSound3DBuffer)");
-
-		pDS3dBuffer->SetMinDistance(g.s.fDXMinDistance, MY_DEFERRED);
-		pDS3dBuffer->SetMaxDistance(g.s.fDXMaxDistance, MY_DEFERRED);
+	if (FAILED(hr = pDSBOutput->SetVolume(lroundf(log10f(g.s.fVolume)*5000.0f)))) {
+		qWarning("DXAudioOutputPlayer: Failed to set volume");
+		return false;
 	}
 
+	if (FAILED(hr = pDSBOutput->QueryInterface(IID_IDirectSoundNotify, reinterpret_cast<void **>(&pDSNotify)))) {
+		qWarning("DXAudioOutputPlayer: QueryInterface (Notify)");
+		return false;
+	}
 
+	if (FAILED(hr = pDSNotify->SetNotificationPositions(NBLOCKS, aPosNotify))) {
+		qWarning("DXAudioOutputPlayer: SetNotificationPositions");
+		return false;
+	}
 
+	if (dxAudio->p3DListener) {
+		if (FAILED(pDSBOutput->QueryInterface(IID_IDirectSound3DBuffer8, reinterpret_cast<void **>(&pDS3dBuffer)))) {
+			qWarning("DXAudioOutputPlayer: QueryInterface (DirectSound3DBuffer)");
+			return false;
+		}
+		pDS3dBuffer->SetMinDistance(g.s.fAudioMinDistance, MY_DEFERRED);
+		pDS3dBuffer->SetMaxDistance(g.s.fAudioMaxDistance, MY_DEFERRED);
+	}
 	qWarning("DXAudioOutputPlayer: %s: New %dHz output buffer of %ld bytes", qPrintable(aop->qsName), SAMPLE_RATE, dsbd.dwBufferBytes);
+	return true;
 }
 
 DXAudioOutputPlayer::~DXAudioOutputPlayer() {
-	qWarning("DXAudioOutputPlayer: %s: Removed", qPrintable(aop->qsName));
+	qWarning("DXAudioOutputPlayer: Removed");
 	if (pDS3dBuffer)
 		pDS3dBuffer->Release();
 	if (pDSNotify)
@@ -221,10 +216,16 @@ bool DXAudioOutputPlayer::playFrames() {
 	DWORD dwApply = MY_DEFERRED;
 
 	if (! pDSBOutput) {
-		setupAudioDevice();
+		if (! setupAudioDevice()) {
+			dxAudio->bOk = false;
+			return false;
+		}
 
-		if (FAILED(hr = pDSBOutput->Lock(0, 0, &aptr1, &nbytes1, &aptr2, &nbytes2, DSBLOCK_ENTIREBUFFER)))
-			qFatal("DXAudioOutputPlayer: Initial Lock");
+		if (FAILED(hr = pDSBOutput->Lock(0, 0, &aptr1, &nbytes1, &aptr2, &nbytes2, DSBLOCK_ENTIREBUFFER))) {
+			qWarning("DXAudioOutputPlayer: Initial Lock");
+			dxAudio->bOk = false;
+			return false;
+		}
 
 		dwBufferSize = nbytes1 + nbytes2;
 		if (aptr1)
@@ -232,8 +233,11 @@ bool DXAudioOutputPlayer::playFrames() {
 		if (aptr2)
 			ZeroMemory(aptr2, nbytes2);
 
-		if (FAILED(hr = pDSBOutput->Unlock(aptr1, nbytes1, aptr2, nbytes2)))
-			qFatal("DXAudioOutputPlayer: Initial Unlock");
+		if (FAILED(hr = pDSBOutput->Unlock(aptr1, nbytes1, aptr2, nbytes2))) {
+			qWarning("DXAudioOutputPlayer: Initial Unlock");
+			dxAudio->bOk = false;
+			return false;
+		}
 
 		dwApply = DS3D_IMMEDIATE;
 
@@ -241,14 +245,19 @@ bool DXAudioOutputPlayer::playFrames() {
 		dwLastPlayPos = 0;
 		dwTotalPlayPos = 0;
 
-		iLastwriteblock = (NBLOCKS - 1 + g.s.iDXOutputDelay) % NBLOCKS;
+		iLastwriteblock = (NBLOCKS - 1 + g.s.iOutputDelay) % NBLOCKS;
 	}
 
-	if (FAILED(hr = pDSBOutput->GetCurrentPosition(&dwPlayPosition, &dwWritePosition)))
-		qFatal("DXAudioOutputPlayer: GetCurrentPosition");
+	if (FAILED(hr = pDSBOutput->GetCurrentPosition(&dwPlayPosition, &dwWritePosition))) {
+		qWarning("DXAudioOutputPlayer: GetCurrentPosition");
+		dxAudio->bOk = false;
+		return false;
+	}
 
 	playblock = dwWritePosition / iByteSize;
-	nowriteblock = (playblock + g.s.iDXOutputDelay + 1) % NBLOCKS;
+	nowriteblock = (playblock + g.s.iOutputDelay + 1) % NBLOCKS;
+
+	STACKVAR(short, psBuffer, 320);
 
 	for (int block=(iLastwriteblock + 1) % NBLOCKS;alive && (block!=nowriteblock);block=(block + 1) % NBLOCKS) {
 
@@ -268,8 +277,11 @@ bool DXAudioOutputPlayer::playFrames() {
 
 		iLastwriteblock = block;
 
-		alive = aop->decodeNextFrame();
-//		qWarning("Block %02d/%02d nowrite %02d, last %02d (Pos %08d / %08d, Del %d)", block, NBLOCKS, nowriteblock, iLastwriteblock, dwPlayPosition, dwWritePosition,g.s.iDXOutputDelay);
+		alive = aop->needSamples(320);
+		for (int i=0;i<320;i++)
+			psBuffer[i] = static_cast<short>(aop->pfBuffer[i]);
+
+//		qWarning("Block %02d/%02d nowrite %02d, last %02d (Pos %08d / %08d, Del %d)", block, NBLOCKS, nowriteblock, iLastwriteblock, dwPlayPosition, dwWritePosition,g.s.iOutputDelay);
 		if (! alive) {
 			iMissingFrames++;
 			// Give 5 seconds grace before killing off buffer, as it seems continously creating and destroying them
@@ -300,33 +312,50 @@ bool DXAudioOutputPlayer::playFrames() {
 			} else {
 				if (mode != DS3DMODE_NORMAL)
 					pDS3dBuffer->SetMode(DS3DMODE_NORMAL, dwApply);
+#ifdef AUDIO_TEST
+				pDS3dBuffer->SetPosition(0, 0, 0, dwApply);
+				qWarning("SetPos %f %f %f", aop->fPos[0], aop->fPos[1], aop->fPos[2]);
+#else
 				pDS3dBuffer->SetPosition(aop->fPos[0], aop->fPos[1], aop->fPos[2], dwApply);
+#endif
 			}
 		}
 
-		if (FAILED(hr = pDSBOutput->Lock(block * iByteSize, iByteSize, &aptr1, &nbytes1, &aptr2, &nbytes2, 0)))
-			qFatal("DXAudioOutput: Lock block %u (%d bytes)",block, iByteSize);
+		if (FAILED(hr = pDSBOutput->Lock(block * iByteSize, iByteSize, &aptr1, &nbytes1, &aptr2, &nbytes2, 0))) {
+			qWarning("DXAudioOutput: Lock block %u (%d bytes)",block, iByteSize);
+			dxAudio->bOk = false;
+			return false;
+		}
 		if (aptr1 && nbytes1)
-			CopyMemory(aptr1, aop->psBuffer, MIN(iByteSize, nbytes1));
+			CopyMemory(aptr1, psBuffer, MIN(iByteSize, nbytes1));
 		if (aptr2 && nbytes2)
-			CopyMemory(aptr2, aop->psBuffer+(nbytes1/2), MIN(iByteSize-nbytes1, nbytes2));
-		if (FAILED(hr = pDSBOutput->Unlock(aptr1, nbytes1, aptr2, nbytes2)))
-			qFatal("DXAudioOutput: Unlock %p(%lu) %p(%lu)",aptr1,nbytes1,aptr2,nbytes2);
+			CopyMemory(aptr2, psBuffer+(nbytes1/2), MIN(iByteSize-nbytes1, nbytes2));
+		if (FAILED(hr = pDSBOutput->Unlock(aptr1, nbytes1, aptr2, nbytes2))) {
+			qWarning("DXAudioOutput: Unlock %p(%lu) %p(%lu)",aptr1,nbytes1,aptr2,nbytes2);
+			dxAudio->bOk = false;
+			return false;
+		}
 
 		// If we get another while we're working, we're already taking care of it.
 		ResetEvent(hNotificationEvent);
 
-		if (FAILED(hr = pDSBOutput->GetCurrentPosition(&dwPlayPosition, &dwWritePosition)))
-			qFatal("DXAudioOutputPlayer: GetCurrentPosition");
+		if (FAILED(hr = pDSBOutput->GetCurrentPosition(&dwPlayPosition, &dwWritePosition))) {
+			qWarning("DXAudioOutputPlayer: GetCurrentPosition");
+			dxAudio->bOk = false;
+			return false;
+		}
 
 		playblock = dwWritePosition / iByteSize;
-		nowriteblock = (playblock + g.s.iDXOutputDelay + 1) % NBLOCKS;
+		nowriteblock = (playblock + g.s.iOutputDelay + 1) % NBLOCKS;
 	}
 
 
 	if (! bPlaying) {
-		if (FAILED(hr = pDSBOutput->Play(0, 0, DSBPLAY_LOOPING)))
-			qFatal("DXAudioOutputPlayer: Play");
+		if (FAILED(hr = pDSBOutput->Play(0, 0, DSBPLAY_LOOPING))) {
+			qWarning("DXAudioOutputPlayer: Play");
+			dxAudio->bOk = false;
+			return false;
+		}
 		bPlaying = true;
 	}
 
@@ -339,6 +368,8 @@ DXAudioOutput::DXAudioOutput() {
 	DSBUFFERDESC        dsbdesc;
 	WAVEFORMATEX wfx;
 	WAVEFORMATEX wfxSet;
+	int ns = 0;
+	unsigned int chanmasks[32];
 
 	pDS = NULL;
 
@@ -349,7 +380,7 @@ DXAudioOutput::DXAudioOutput() {
 	ZeroMemory(&dsbdesc, sizeof(DSBUFFERDESC));
 	dsbdesc.dwSize  = sizeof(DSBUFFERDESC);
 	dsbdesc.dwFlags = DSBCAPS_PRIMARYBUFFER;
-	if (g.s.a3dModel != Settings::No3D)
+	if (g.s.bPositionalAudio)
 		dsbdesc.dwFlags |= DSBCAPS_CTRL3D;
 
 	ZeroMemory(&wfxSet, sizeof(wfxSet));
@@ -384,14 +415,14 @@ DXAudioOutput::DXAudioOutput() {
 	else if (FAILED(hr = pDS->CreateSoundBuffer(&dsbdesc, &pDSBPrimary, NULL)))
 		qWarning("DXAudioOutput: CreateSoundBuffer (Primary) : 0x%08lx", hr);
 	else if (FAILED(hr = pDSBPrimary->SetFormat(&wfx)))
-		qFatal("DXAudioOutput: SetFormat");
+		qWarning("DXAudioOutput: SetFormat");
 	else if (FAILED(hr = pDSBPrimary->GetFormat(&wfxSet, sizeof(wfxSet), NULL)))
-		qFatal("DXAudioOutput: GetFormat");
-	else if (g.s.a3dModel != Settings::No3D) {
+		qWarning("DXAudioOutput: GetFormat");
+	else if (g.s.bPositionalAudio) {
 		if (FAILED(hr = pDSBPrimary->QueryInterface(IID_IDirectSound3DListener8, reinterpret_cast<void **>(&p3DListener)))) {
 			qWarning("DXAudioOutput: QueryInterface (DirectSound3DListener8): 0x%08lx",hr);
 		} else {
-			p3DListener->SetRolloffFactor(g.s.fDXRollOff, MY_DEFERRED);
+			p3DListener->SetRolloffFactor(g.s.fAudioRollOff, MY_DEFERRED);
 			p3DListener->SetDopplerFactor(DS3D_MINDOPPLERFACTOR, MY_DEFERRED);
 			p3DListener->CommitDeferredSettings();
 			bOk = true;
@@ -401,17 +432,67 @@ DXAudioOutput::DXAudioOutput() {
 	}
 
 	if (! bOk) {
-		QMessageBox::warning(NULL, tr("Mumble"), tr("Opening chosen DirectSound Output failed. No audio will be heard."), QMessageBox::Ok, QMessageBox::NoButton);
+		g.mw->msgBox(tr("Opening chosen DirectSound Output failed. No audio will be heard."));
 		return;
 	}
 
 	if (failed)
-		QMessageBox::warning(NULL, tr("Mumble"), tr("Opening chosen DirectSound Output failed. Default device will be used."), QMessageBox::Ok, QMessageBox::NoButton);
+		g.mw->msgBox(tr("Opening chosen DirectSound Output failed. Default device will be used."));
 
 	qWarning("DXAudioOutput: Primary buffer of %ld Hz, %d channels, %d bits",wfxSet.nSamplesPerSec,wfxSet.nChannels,wfxSet.wBitsPerSample);
 	if (p3DListener)
 		qWarning("DXAudioOutput: 3D mode active");
 
+	DWORD dwSpeakerConfig;
+	pDS->GetSpeakerConfig(&dwSpeakerConfig);
+
+	DWORD dwMask = 0;
+	bool bHead = false;
+
+	switch (DSSPEAKER_CONFIG(dwSpeakerConfig)) {
+		case DSSPEAKER_HEADPHONE:
+			dwMask = KSAUDIO_SPEAKER_STEREO;
+			bHead = true;
+			break;
+		case DSSPEAKER_MONO:
+			dwMask = KSAUDIO_SPEAKER_MONO;
+			break;
+		case DSSPEAKER_QUAD:
+			dwMask = KSAUDIO_SPEAKER_QUAD;
+			break;
+		case DSSPEAKER_STEREO:
+			dwMask = KSAUDIO_SPEAKER_STEREO;
+			break;
+		case DSSPEAKER_SURROUND:
+			dwMask = KSAUDIO_SPEAKER_SURROUND;
+			break;
+		case DSSPEAKER_5POINT1:
+			dwMask = KSAUDIO_SPEAKER_5POINT1;
+			break;
+		case DSSPEAKER_7POINT1:
+			dwMask = KSAUDIO_SPEAKER_7POINT1;
+			break;
+		case DSSPEAKER_7POINT1_SURROUND:
+			dwMask = KSAUDIO_SPEAKER_7POINT1_SURROUND;
+			break;
+		case DSSPEAKER_5POINT1_SURROUND:
+			dwMask = KSAUDIO_SPEAKER_5POINT1_SURROUND;
+			break;
+		default:
+			dwMask = 0;
+			break;
+	}
+
+	for (int i=0;i<32;i++) {
+		if (dwMask & (1 << i)) {
+			chanmasks[ns++] = 1 << i;
+		}
+	}
+
+	iMixerFreq = SAMPLE_RATE;
+	iChannels = ns;
+	initializeMixer(chanmasks, bHead);
+
 	bRunning = true;
 }
 
@@ -445,7 +526,7 @@ void DXAudioOutput::updateListener() {
 	DS3DLISTENER li;
 	Plugins *p = g.p;
 	li.dwSize=sizeof(li);
-	if (p->bValid && ! g.bCenterPosition && p3DListener) {
+	if (p->fetch() && ! g.bCenterPosition && p3DListener) {
 		// Only set this if we need to. If centerposition is on, or we don't have valid data,
 		// the 3d mode for the buffers will be disabled, so don't bother with updates.
 		p3DListener->SetPosition(p->fPosition[0], p->fPosition[1], p->fPosition[2], MY_DEFERRED);
@@ -454,6 +535,9 @@ void DXAudioOutput::updateListener() {
 	}
 	if (FAILED(hr =p3DListener->CommitDeferredSettings()))
 		qWarning("DXAudioOutputPlayer: CommitDeferrredSettings failed 0x%08lx", hr);
+#ifdef AUDIO_TEST
+	qWarning("SetLis %f %f %f", p->fPosition[0], p->fPosition[1], p->fPosition[2]);
+#endif
 
 	/*
 		float a[3], b[3];
@@ -479,9 +563,8 @@ void DXAudioOutput::run() {
 	LARGE_INTEGER ticksPerSecond;
 	LARGE_INTEGER ticksPerFrame;
 	LARGE_INTEGER ticksNow;
-LARGE_INTEGER ticksNext = {QuadPart:
-	                           0L
-	                          };
+	LARGE_INTEGER ticksNext;
+	ticksNext.QuadPart = 0LL;
 
 	bool found = false;
 	bool alive = false;
@@ -489,12 +572,8 @@ LARGE_INTEGER ticksNext = {QuadPart:
 	QueryPerformanceFrequency(&ticksPerSecond);
 	ticksPerFrame.QuadPart = ticksPerSecond.QuadPart / 50;
 
-	while (bRunning) {
-		// Ok, so I thought about it..
-		// We might optimize this by having a QHash<hEvent, AOP *>, but..
-		// .. most of the time, there will be one or maybe two AOPs active.
-		// The overhead of updating the cache is likely to outstrip the
-		// benefit.
+	while (bRunning && bOk) {
+		bOk = true;
 
 		count = 0;
 
@@ -530,5 +609,8 @@ LARGE_INTEGER ticksNext = {QuadPart:
 		if (found && ! alive)
 			removeBuffer(dxaop->aop);
 	}
+	if (! bOk) {
+		g.mw->msgBox(tr("Lost DirectSound output device."));
+	}
 }
 
