@@ -38,7 +38,7 @@ static ConfigWidget *GlobalShortcutConfigDialogNew(Settings &st) {
 	return new GlobalShortcutConfig(st);
 }
 
-static ConfigRegistrar registrar(12, GlobalShortcutConfigDialogNew);
+static ConfigRegistrar registrar(1200, GlobalShortcutConfigDialogNew);
 
 ShortcutKeyWidget::ShortcutKeyWidget(QWidget *p) : QLineEdit(p) {
 	setReadOnly(true);
@@ -105,6 +105,7 @@ void ShortcutKeyWidget::displayKeys() {
 }
 
 GlobalShortcutConfig::GlobalShortcutConfig(Settings &st) : ConfigWidget(st) {
+	setObjectName(QLatin1String("GlobalShortcutConfig"));
 	QGroupBox *qgbShortcuts = new QGroupBox(tr("Shortcuts"));
 	QLabel *lab;
 
@@ -172,6 +173,7 @@ void GlobalShortcutConfig::save() const {
 
 void GlobalShortcutConfig::accept() const {
 	GlobalShortcutEngine::engine->bNeedRemap = true;
+	GlobalShortcutEngine::engine->needRemap();
 }
 
 bool GlobalShortcutConfig::expert(bool) {
@@ -181,6 +183,7 @@ bool GlobalShortcutConfig::expert(bool) {
 
 GlobalShortcutEngine::GlobalShortcutEngine(QObject *p) : QThread(p) {
 	bNeedRemap = true;
+	needRemap();
 }
 
 void GlobalShortcutEngine::remap() {
@@ -213,6 +216,8 @@ void GlobalShortcutEngine::resetMap() {
 	qlActiveButtons.clear();
 }
 
+void GlobalShortcutEngine::needRemap() {
+}
 
 void GlobalShortcutEngine::handleButton(const QVariant &button, bool down) {
 	if (tReset.elapsed() > 100000) {
@@ -256,11 +261,13 @@ void GlobalShortcutEngine::add(GlobalShortcut *gs) {
 
 	GlobalShortcutEngine::engine->qmShortcuts.insert(gs->idx, gs);
 	GlobalShortcutEngine::engine->bNeedRemap = true;
+	GlobalShortcutEngine::engine->needRemap();
 }
 
 void GlobalShortcutEngine::remove(GlobalShortcut *gs) {
 	engine->qmShortcuts.remove(gs->idx);
 	engine->bNeedRemap = true;
+	engine->needRemap();
 	if (engine->qmShortcuts.isEmpty()) {
 		delete engine;
 		GlobalShortcutEngine::engine = NULL;
