@@ -56,11 +56,76 @@ MessageBoxEvent::MessageBoxEvent(QString m) : QEvent(static_cast<QEvent::Type>(M
 	msg = m;
 }
 
+LogTitleBar::LogTitleBar() {
+	qtTick = new QTimer(this);
+	qtTick->setSingleShot(true);
+	connect(qtTick, SIGNAL(timeout()), this, SLOT(tick()));
+	size = newsize = 1;
+}
+
+QSize LogTitleBar::sizeHint() const {
+	return minimumSizeHint();
+}
+
+QSize LogTitleBar::minimumSizeHint() const {
+	return QSize(size,size);
+}
+
+bool LogTitleBar::eventFilter(QObject *, QEvent *evt) {
+	QDockWidget *qdw = qobject_cast<QDockWidget*>(parentWidget());
+
+	switch (evt->type()) {
+		case QEvent::Leave:
+		case QEvent::Enter:
+		case QEvent::MouseMove:
+		case QEvent::MouseButtonRelease: {
+				newsize = 0;
+				QPoint p = qdw->mapFromGlobal(QCursor::pos());
+				if ((p.x() >= 0) && (p.x() < qdw->width())  && (p.y() >= 0) && (p.y() < 15))
+					newsize = 15;
+				if (newsize > 0 && !qtTick->isActive())
+					qtTick->start(500);
+				else if ((newsize == size) && qtTick->isActive())
+					qtTick->stop();
+				else if (newsize == 0)
+					tick();
+			}
+		default:
+			break;
+	}
+
+	return false;
+}
+
+void LogTitleBar::tick() {
+	QDockWidget *qdw = qobject_cast<QDockWidget*>(parentWidget());
+
+	if (newsize == size)
+		return;
+
+	size = newsize;
+	qdw->setTitleBarWidget(this);
+}
+
 MainWindow::MainWindow(QWidget *p) : QMainWindow(p) {
+
+	qiIcon.addFile(QLatin1String("skin:mumble.16x16.png"));
+	qiIcon.addFile(QLatin1String("skin:mumble.32x32.png"));
+	qiIcon.addFile(QLatin1String("skin:mumble.64x64.png"));
+
+	// Set application icon except on MacOSX, where the window-icon
+	// shown in the title-bar usually serves as a draggable version of the
+	// current open document (i.e. you can copy the open document anywhere
+	// simply by dragging this icon).
+#ifndef Q_OS_MAC
+	qApp->setWindowIcon(qiIcon);
+#endif
+
 	Channel::add(0, tr("Root"), NULL);
 
 	aclEdit = NULL;
 	banEdit = NULL;
+	bNoHide = false;
 
 	qtReconnect = new QTimer(this);
 	qtReconnect->setInterval(10000);
@@ -73,6 +138,11 @@ MainWindow::MainWindow(QWidget *p) : QMainWindow(p) {
 
 	connect(g.sh, SIGNAL(connected()), this, SLOT(serverConnected()));
 	connect(g.sh, SIGNAL(disconnected(QString)), this, SLOT(serverDisconnected(QString)));
+
+	// Fix context of all actions.
+	QList<QAction *> qla = findChildren<QAction *>();
+	foreach(QAction *a, qla)
+	a->setShortcutContext(Qt::ApplicationShortcut);
 }
 
 void MainWindow::createActions() {
@@ -124,13 +194,29 @@ void MainWindow::createActions() {
 	gsAltTalk=new GlobalShortcut(this, idx++, tr("Alt Push-to-Talk", "Global Shortcut"));
 	gsAltTalk->setObjectName(QLatin1String("AltPushToTalk"));
 
-	qstiIcon = new QSystemTrayIcon(qApp->windowIcon(), this);
+	gsMinimal=new GlobalShortcut(this, idx++, tr("Toggle Minimal", "Global Shortcut"));
+	gsMinimal->setObjectName(QLatin1String("ToggleMinimal"));
+
+	qstiIcon = new QSystemTrayIcon(qiIcon, this);
+	qstiIcon->setToolTip(tr("Mumble"));
 	qstiIcon->setObjectName(QLatin1String("Icon"));
+#ifndef Q_OS_MAC
 	qstiIcon->show();
+#endif
 }
 
 void MainWindow::setupGui()  {
 	setWindowTitle(tr("Mumble -- %1").arg(QLatin1String(MUMBLE_RELEASE)));
+	setCentralWidget(qtvPlayers);
+
+#ifdef Q_OS_MAC
+	QMenu *qmWindow = new QMenu(tr("&Window"), this);
+	menubar->insertMenu(qmHelp->menuAction(), qmWindow);
+	qmWindow->addAction(tr("Minimize"), this, SLOT(showMinimized()), QKeySequence(tr("Ctrl+M")));
+	qmWindow->addAction(tr("Close"), this, SLOT(on_qaQuit_triggered()), QKeySequence(tr("Ctrl+W")));
+#endif
+
+	qteLog->document()->setDefaultStyleSheet(qApp->styleSheet());
 
 	pmModel = new PlayerModel(qtvPlayers);
 
@@ -144,31 +230,34 @@ void MainWindow::setupGui()  {
 	qaAudioMute->setChecked(g.s.bMute);
 	qaAudioDeaf->setChecked(g.s.bDeaf);
 	qaAudioTTS->setChecked(g.s.bTTS);
-	qaAudioLocalDeafen->setChecked(g.s.bLocalDeafen);
 	qaHelpWhatsThis->setShortcuts(QKeySequence::WhatsThis);
 
+	qaConfigMinimal->setChecked(g.s.bMinimalView);
+
 	connect(gsResetAudio, SIGNAL(down()), qaAudioReset, SLOT(trigger()));
 	connect(gsMuteSelf, SIGNAL(down()), qaAudioMute, SLOT(trigger()));
 	connect(gsDeafSelf, SIGNAL(down()), qaAudioDeaf, SLOT(trigger()));
-
 	connect(gsUnlink, SIGNAL(down()), qaAudioUnlink, SLOT(trigger()));
+	connect(gsMinimal, SIGNAL(down()), qaConfigMinimal, SLOT(trigger()));
 
-	if (g.s.bHorizontal) {
-		qsSplit->setOrientation(Qt::Horizontal);
-		qsSplit->addWidget(qteLog);
-		qsSplit->addWidget(qtvPlayers);
-	} else {
-		qsSplit->setOrientation(Qt::Vertical);
-		qsSplit->addWidget(qtvPlayers);
-		qsSplit->addWidget(qteLog);
+	ltbDockTitle = new LogTitleBar();
+	qdwLog->setTitleBarWidget(ltbDockTitle);
+
+	foreach(QWidget *w, qdwLog->findChildren<QWidget *>()) {
+		w->installEventFilter(ltbDockTitle);
+		w->setMouseTracking(true);
 	}
 
-	setCentralWidget(qsSplit);
-
 	restoreGeometry(g.s.qbaMainWindowGeometry);
 	restoreState(g.s.qbaMainWindowState);
-	qsSplit->restoreState(g.s.qbaSplitterState);
 	qtvPlayers->header()->restoreState(g.s.qbaHeaderState);
+
+	setupView();
+}
+
+MainWindow::~MainWindow() {
+	delete pmModel;
+	delete qtvPlayers;
 }
 
 void MainWindow::msgBox(QString msg) {
@@ -177,6 +266,7 @@ void MainWindow::msgBox(QString msg) {
 }
 
 void MainWindow::closeEvent(QCloseEvent *e) {
+#ifndef Q_OS_MAC
 	if (g.sh && g.sh->isRunning()) {
 		QMessageBox mb(QMessageBox::Warning, tr("Mumble"), tr("Mumble is currently connected to a server. Do you want to Close or Minimize it?"), QMessageBox::NoButton, this);
 		QPushButton *qpbClose = mb.addButton(tr("Close"), QMessageBox::YesRole);
@@ -189,19 +279,23 @@ void MainWindow::closeEvent(QCloseEvent *e) {
 			return;
 		}
 	}
+#endif
 	g.uiSession = 0;
 	g.s.qbaMainWindowGeometry = saveGeometry();
 	g.s.qbaMainWindowState = saveState();
-	g.s.qbaSplitterState = qsSplit->saveState();
-	g.s.qbaHeaderState = qtvPlayers->header()->saveState();
+	if (! g.s.bMinimalView) {
+		g.s.qbaHeaderState = qtvPlayers->header()->saveState();
+	}
 	QMainWindow::closeEvent(e);
 	qApp->quit();
 }
 
 void MainWindow::hideEvent(QHideEvent *e) {
-	if (qstiIcon->isSystemTrayAvailable())
+#ifndef Q_OS_MAC
+	if (!bNoHide && qstiIcon->isSystemTrayAvailable())
 		qApp->postEvent(this, new QEvent(static_cast<QEvent::Type>(TI_QEVENT)));
 	QMainWindow::hideEvent(e);
+#endif
 }
 
 void MainWindow::on_qtvPlayers_customContextMenuRequested(const QPoint &mpos) {
@@ -245,6 +339,20 @@ void MainWindow::openUrl(const QUrl &url) {
 	QString pw = url.password();
 	qsDesiredChannel = url.path();
 
+	if (g.sh && g.sh->isRunning()) {
+		QString oHost, oUser, oPw;
+		int oport;
+		g.sh->getConnectionInfo(oHost, oport, oUser, oPw);
+
+		if ((user.isEmpty() || (user == oUser)) &&
+		        (host.isEmpty() || ((host == oHost) && (port == oport)))) {
+			findDesiredChannel();
+			return;
+		}
+	}
+
+	Database::fuzzyMatch(user, pw, host, port);
+
 	if (user.isEmpty()) {
 		bool ok;
 		user = QInputDialog::getText(this, tr("Connecting to %1").arg(url.toString()), tr("Enter username"), QLineEdit::Normal, g.s.qsUsername, &ok);
@@ -264,6 +372,66 @@ void MainWindow::openUrl(const QUrl &url) {
 	g.sh->start(QThread::TimeCriticalPriority);
 }
 
+void MainWindow::findDesiredChannel() {
+	bool found = false;
+	QStringList qlChans = qsDesiredChannel.split(QLatin1String("/"));
+	Channel *chan = Channel::get(0);
+	QString str = QString();
+	while (chan && qlChans.count() > 0) {
+		QString elem = qlChans.takeFirst().toLower();
+		if (elem.isEmpty())
+			continue;
+		if (str.isNull())
+			str = elem;
+		else
+			str = str + QLatin1String("/") + elem;
+		foreach(Channel *c, chan->qlChannels) {
+			if (c->qsName.toLower() == str) {
+				str = QString();
+				found = true;
+				chan = c;
+				break;
+			}
+		}
+	}
+	if (found && (chan != ClientPlayer::get(g.uiSession)->cChannel)) {
+		MessagePlayerMove mpm;
+		mpm.uiVictim = g.uiSession;
+		mpm.iChannelId = chan->iId;
+		g.sh->sendMessage(&mpm);
+	}
+}
+
+void MainWindow::setupView() {
+	bool showit = ! g.s.bMinimalView;
+
+	bNoHide = true;
+
+	if (! showit)
+		g.s.qbaMainWindowState = saveState();
+
+	qdwLog->setVisible(showit);
+
+	if (showit) {
+		restoreState(g.s.qbaMainWindowState);
+		qdwLog->setVisible(showit);
+	}
+
+
+	Qt::WindowFlags f = windowFlags();
+	if (showit)
+		f = 0;
+	else
+		f = Qt::CustomizeWindowHint;
+	setWindowFlags(f);
+	show();
+	qtvPlayers->header()->setVisible(showit);
+	menuBar()->setVisible(showit);
+
+
+	bNoHide = false;
+}
+
 void MainWindow::on_qaServerConnect_triggered() {
 	ConnectDialog *cd = new ConnectDialog(this);
 	int res = cd->exec();
@@ -355,6 +523,31 @@ void MainWindow::on_qaServerInformation_triggered() {
 
 void MainWindow::on_qmPlayer_aboutToShow() {
 	Player *p = pmModel->getPlayer(qtvPlayers->currentIndex());
+	bool self = p && (p->uiSession == g.uiSession);
+
+	qmPlayer->clear();
+
+	qmPlayer->addAction(qaPlayerKick);
+	qmPlayer->addAction(qaPlayerBan);
+	qmPlayer->addAction(qaPlayerMute);
+	qmPlayer->addAction(qaPlayerDeaf);
+	qmPlayer->addAction(qaPlayerLocalMute);
+	qmPlayer->addAction(qaPlayerTextMessage);
+
+	if (self) {
+		qmPlayer->addSeparator();
+		qmPlayer->addAction(qaAudioMute);
+		qmPlayer->addAction(qaAudioDeaf);
+	}
+
+	if (g.s.bMinimalView) {
+		qmPlayer->addSeparator();
+		qmPlayer->addMenu(qmServer);
+		qmPlayer->addMenu(qmAudio);
+		qmPlayer->addMenu(qmConfig);
+		qmPlayer->addMenu(qmHelp);
+	}
+
 	if (! p) {
 		qaPlayerKick->setEnabled(false);
 		qaPlayerBan->setEnabled(false);
@@ -363,15 +556,16 @@ void MainWindow::on_qmPlayer_aboutToShow() {
 		qaPlayerDeaf->setEnabled(false);
 		qaPlayerTextMessage->setEnabled(false);
 	} else {
-		qaPlayerKick->setEnabled(true);
-		qaPlayerBan->setEnabled(true);
-		qaPlayerMute->setEnabled(true);
-		qaPlayerLocalMute->setEnabled(p->uiSession != g.uiSession);
-		qaPlayerDeaf->setEnabled(true);
-		qaPlayerMute->setChecked(p->bMute);
-		qaPlayerLocalMute->setChecked(p->bLocalMute);
-		qaPlayerDeaf->setChecked(p->bDeaf);
+		qaPlayerKick->setEnabled(! self);
+		qaPlayerBan->setEnabled(! self);
+		qaPlayerMute->setEnabled(! self || p->bMute);
+		qaPlayerDeaf->setEnabled(! self || p->bDeaf);
 		qaPlayerTextMessage->setEnabled(true);
+		qaPlayerLocalMute->setEnabled(! self);
+
+		qaPlayerMute->setChecked(p->bMute);
+		qaPlayerDeaf->setChecked(p->bDeaf);
+		qaPlayerLocalMute->setChecked(p->bLocalMute);
 	}
 }
 
@@ -482,8 +676,28 @@ void MainWindow::on_qaQuit_triggered() {
 void MainWindow::on_qmChannel_aboutToShow() {
 	QModelIndex idx = qtvPlayers->currentIndex();
 
-	bool add, remove, acl, rename, link, unlink, unlinkall, msg;
+	qmChannel->clear();
+	qmChannel->addAction(qaChannelAdd);
+	qmChannel->addAction(qaChannelRemove);
+	qmChannel->addAction(qaChannelACL);
+	qmChannel->addAction(qaChannelRename);
+	qmChannel->addSeparator();
+	qmChannel->addAction(qaChannelLink);
+	qmChannel->addAction(qaChannelUnlink);
+	qmChannel->addAction(qaChannelUnlinkAll);
+	qmChannel->addSeparator();
+	qmChannel->addAction(qaChannelSendMessage);
+	qmChannel->addAction(qaChannelSendTreeMessage);
 
+	if (g.s.bMinimalView) {
+		qmChannel->addSeparator();
+		qmChannel->addMenu(qmServer);
+		qmChannel->addMenu(qmAudio);
+		qmChannel->addMenu(qmConfig);
+		qmChannel->addMenu(qmHelp);
+	}
+
+	bool add, remove, acl, rename, link, unlink, unlinkall, msg;
 	add = remove = acl = rename = link = unlink = unlinkall = msg = false;
 
 	if (g.uiSession != 0) {
@@ -728,14 +942,6 @@ void MainWindow::on_qaAudioDeaf_triggered() {
 	g.sh->sendMessage(&mpsmd);
 }
 
-void MainWindow::on_qaAudioLocalDeafen_triggered() {
-	g.s.bLocalDeafen = qaAudioLocalDeafen->isChecked();
-	if (g.s.bLocalDeafen) {
-		QMessageBox::information(this, tr("Mumble"), tr("You are now in local deafen mode. This mode is not reflected on the server, and you will still be transmitting "
-		                         "voice to the server. This mode should only be used if there are several people in the same room and one of them have Mumble on loudspeakers."));
-	}
-}
-
 void MainWindow::on_qaAudioTTS_triggered() {
 	g.s.bTTS = qaAudioTTS->isChecked();
 }
@@ -754,9 +960,15 @@ void MainWindow::on_qaConfigDialog_triggered() {
 	dlg.exec();
 }
 
+void MainWindow::on_qaConfigMinimal_triggered() {
+	g.s.bMinimalView = qaConfigMinimal->isChecked();
+	setupView();
+}
+
 void MainWindow::on_qaAudioWizard_triggered() {
-	AudioWizard aw;
-	aw.exec();
+	AudioWizard *aw = new AudioWizard(this);
+	aw->exec();
+	delete aw;
 }
 
 void MainWindow::on_qaHelpWhatsThis_triggered() {
@@ -1013,7 +1225,6 @@ void MainWindow::on_Icon_activated(QSystemTrayIcon::ActivationReason) {
 	}
 }
 
-
 void MainWindow::customEvent(QEvent *evt) {
 	if (evt->type() == TI_QEVENT) {
 		hide();
