@@ -52,16 +52,35 @@ class MessageBoxEvent : public QEvent {
 		MessageBoxEvent(QString msg);
 };
 
+class LogTitleBar : public QWidget {
+		Q_OBJECT
+	protected:
+		QTimer *qtTick;
+		int size;
+		int newsize;
+	public:
+		LogTitleBar();
+		QSize sizeHint() const;
+		QSize minimumSizeHint() const;
+	public slots:
+		void tick();
+	protected:
+		bool eventFilter(QObject *, QEvent *);
+};
+
 class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWindow {
 		friend class PlayerModel;
 		Q_OBJECT
 	public:
 		PlayerModel *pmModel;
 		QSystemTrayIcon *qstiIcon;
+		QIcon qiIcon;
 
 		GlobalShortcut *gsPushTalk, *gsResetAudio, *gsMuteSelf, *gsDeafSelf;
 		GlobalShortcut *gsUnlink, *gsCenterPos, *gsPushMute, *gsMetaChannel, *gsToggleOverlay;
 		GlobalShortcut *gsAltTalk;
+		GlobalShortcut *gsMinimal;
+		LogTitleBar *ltbDockTitle;
 
 		ACLEditor *aclEdit;
 		BanEditor *banEdit;
@@ -77,6 +96,9 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		void createActions();
 		void setupGui();
 		void customEvent(QEvent *evt);
+		void findDesiredChannel();
+		void setupView();
+		bool bNoHide;
 		virtual void closeEvent(QCloseEvent *e);
 		virtual void hideEvent(QHideEvent *e);
 	public slots:
@@ -106,11 +128,11 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		void on_qaAudioReset_triggered();
 		void on_qaAudioMute_triggered();
 		void on_qaAudioDeaf_triggered();
-		void on_qaAudioLocalDeafen_triggered();
 		void on_qaAudioTTS_triggered();
 		void on_qaAudioUnlink_triggered();
 		void on_qaAudioStats_triggered();
 		void on_qaConfigDialog_triggered();
+		void on_qaConfigMinimal_triggered();
 		void on_qaAudioWizard_triggered();
 		void on_qaHelpWhatsThis_triggered();
 		void on_qaHelpAbout_triggered();
@@ -133,6 +155,7 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		void openUrl(const QUrl &url);
 	public:
 		MainWindow(QWidget *parent);
+		~MainWindow();
 
 		// From msgHandler. Implementation in Messages.cpp
 		virtual void msgSpeex(Connection *, MessageSpeex *);
