@@ -46,6 +46,7 @@ ModelItem::ModelItem(Channel *c) {
 	this->pPlayer = NULL;
 	c_qhChannels.insert(c, this);
 	parent = c_qhChannels.value(c->cParent);
+	iPlayers = 0;
 }
 
 ModelItem::ModelItem(ClientPlayer *p) {
@@ -53,17 +54,27 @@ ModelItem::ModelItem(ClientPlayer *p) {
 	this->pPlayer = p;
 	c_qhPlayers.insert(p, this);
 	parent = c_qhChannels.value(p->cChannel);
+	iPlayers = 0;
 }
 
 ModelItem::ModelItem(ModelItem *i) {
-	// Create a dummy clone.
+	// Create a full clone
 	this->cChan = i->cChan;
 	this->pPlayer = i->pPlayer;
 	this->parent = i->parent;
+
 	if (pPlayer)
 		c_qhPlayers.insert(pPlayer, this);
 	else if (cChan)
 		c_qhChannels.insert(cChan, this);
+
+	iPlayers = i->iPlayers;
+
+	foreach(ModelItem *c, i->qlChildren) {
+		ModelItem *ni = new ModelItem(c);
+		ni->parent = this;
+		qlChildren << ni;
+	}
 }
 
 ModelItem::~ModelItem() {
@@ -75,6 +86,15 @@ ModelItem::~ModelItem() {
 		c_qhPlayers.remove(pPlayer);
 }
 
+void ModelItem::wipe() {
+	foreach(ModelItem *i, qlChildren) {
+		i->wipe();
+		delete i;
+	}
+	qlChildren.clear();
+	iPlayers = 0;
+}
+
 ModelItem *ModelItem::child(int idx) const {
 	if (! validRow(idx))
 		return NULL;
@@ -490,12 +510,28 @@ void PlayerModel::moveItem(ModelItem *oldparent, ModelItem *newparent, ModelItem
 
 	endInsertRows();
 
+	QStack<ModelItem *> qsOld, qsNew;
+	qsOld << item;
+	qsNew << t;
+	while (! qsOld.isEmpty()) {
+		ModelItem *o = qsOld.pop();
+		ModelItem *n = qsNew.pop();
+
+		Q_ASSERT(o->qlChildren.count() == n->qlChildren.count());
+		for(int i=0;i<n->qlChildren.count();++i) {
+			qsOld << o->qlChildren.at(i);
+			qsNew << n->qlChildren.at(i);
+			changePersistentIndex(createIndex(i, 0, o->qlChildren.at(i)), createIndex(i, 0, n->qlChildren.at(i)));
+		}
+	}
+
 	changePersistentIndex(createIndex(oldrow, 0, item), createIndex(newrow, 0, t));
 
 	beginRemoveRows(index(oldparent), oldrow, oldrow);
 	oldparent->qlChildren.removeAt(oldrow);
 	endRemoveRows();
 
+	item->wipe();
 	delete item;
 
 	if (active.isValid()) {
@@ -516,6 +552,17 @@ void PlayerModel::expandAll(Channel *c) {
 	}
 }
 
+void PlayerModel::collapseEmpty(Channel *c) {
+	while (c) {
+		ModelItem *mi = ModelItem::c_qhChannels.value(c);
+		if (mi->iPlayers == 0)
+			g.mw->qtvPlayers->setExpanded(index(c), false);
+		else
+			break;
+		c = c->cParent;
+	}
+}
+
 void PlayerModel::ensureSelfVisible() {
 	if (! g.uiSession)
 		return;
@@ -574,6 +621,8 @@ ClientPlayer *PlayerModel::addPlayer(unsigned int id, const QString &name) {
 	c->addPlayer(p);
 	endInsertRows();
 
+	citem->iPlayers++;
+
 	if (g.uiSession && (p->cChannel == ClientPlayer::get(g.uiSession)->cChannel))
 		updateOverlay();
 
@@ -600,10 +649,19 @@ void PlayerModel::removePlayer(ClientPlayer *p) {
 	ClientPlayer::remove(p);
 	delete p;
 	delete item;
+
+	while (citem) {
+		citem->iPlayers--;
+		citem = citem->parent;
+	}
+
+	if (g.s.ceExpand == Settings::ChannelsWithPlayers)
+		collapseEmpty(c);
 }
 
 void PlayerModel::movePlayer(ClientPlayer *p, Channel *np) {
-	ModelItem *opi = ModelItem::c_qhChannels.value(p->cChannel);
+	Channel *oc = p->cChannel;
+	ModelItem *opi = ModelItem::c_qhChannels.value(oc);
 	ModelItem *pi = ModelItem::c_qhChannels.value(np);
 	ModelItem *item = ModelItem::c_qhPlayers.value(p);
 	moveItem(opi, pi, item);
@@ -612,6 +670,20 @@ void PlayerModel::movePlayer(ClientPlayer *p, Channel *np) {
 		ensureSelfVisible();
 		recheckLinks();
 	}
+
+	while (opi) {
+		opi->iPlayers--;
+		opi = opi->parent;
+	}
+	while (pi) {
+		pi->iPlayers++;
+		pi = pi->parent;
+	}
+
+	if (g.s.ceExpand == Settings::ChannelsWithPlayers) {
+		expandAll(np);
+		collapseEmpty(oc);
+	}
 }
 
 void PlayerModel::renamePlayer(ClientPlayer *p, const QString &name) {
@@ -656,7 +728,7 @@ Channel *PlayerModel::addChannel(int id, Channel *p, const QString &name) {
 	citem->qlChildren.insert(row, item);
 	endInsertRows();
 
-	if (g.s.bExpandAll)
+	if (g.s.ceExpand == Settings::AllChannels)
 		g.mw->qtvPlayers->setExpanded(index(item), true);
 
 	return c;
