@@ -40,19 +40,15 @@
 
 static PulseAudioSystem *pasys = NULL;
 
-static const pa_sample_spec ss = { PA_SAMPLE_S16LE, SAMPLE_RATE, 1 };
-
 #define NBLOCKS 8
 
-#define MAX(a,b)        ( (a) > (b) ? (a) : (b) )
-#define MIN(a,b)        ( (a) < (b) ? (a) : (b) )
-
 class PulseAudioInputRegistrar : public AudioInputRegistrar {
 	public:
 		PulseAudioInputRegistrar();
 		virtual AudioInput *create();
 		virtual const QList<audioDevice> getDeviceChoices();
-		virtual void setDeviceChoice(const QVariant &);
+		virtual void setDeviceChoice(const QVariant &, Settings &);
+		virtual bool canEcho(const QString &);
 };
 
 
@@ -61,16 +57,35 @@ class PulseAudioOutputRegistrar : public AudioOutputRegistrar {
 		PulseAudioOutputRegistrar();
 		virtual AudioOutput *create();
 		virtual const QList<audioDevice> getDeviceChoices();
-		virtual void setDeviceChoice(const QVariant &);
+		virtual void setDeviceChoice(const QVariant &, Settings &);
 };
 
-static PulseAudioInputRegistrar airPulseAudio;
-static PulseAudioOutputRegistrar aorPulseAudio;
-
 class PulseAudioInit : public DeferInit {
 	public:
-		void initialize() { pasys = new PulseAudioSystem(); };
-		void destroy() { delete pasys; pasys = NULL; };
+		PulseAudioInputRegistrar *airPulseAudio;
+		PulseAudioOutputRegistrar *aorPulseAudio;
+		void initialize() {
+			pasys = new PulseAudioSystem();
+			pasys->qmWait.lock();
+			pasys->start(QThread::TimeCriticalPriority);
+			pasys->qwcWait.wait(&pasys->qmWait, 1000);
+			if (pasys->bPulseIsGood) {
+				airPulseAudio = new PulseAudioInputRegistrar();
+				aorPulseAudio = new PulseAudioOutputRegistrar();
+			} else {
+				airPulseAudio = NULL;
+				aorPulseAudio = NULL;
+			}
+			qWarning("Pulse Done! %d", pasys->bPulseIsGood);
+		};
+		void destroy() {
+			if (airPulseAudio)
+				delete airPulseAudio;
+			if (aorPulseAudio)
+				delete aorPulseAudio;
+			delete pasys;
+			pasys = NULL;
+		};
 };
 
 static PulseAudioInit pulseinit;
@@ -79,34 +94,26 @@ PulseAudioSystem::PulseAudioSystem() {
 	pasInput = pasOutput = pasSpeaker = NULL;
 	bSourceDone=bSinkDone=bServerDone = false;
 	iDelayCache = 0;
-
-	psInput = psEcho = NULL;
-	iInputIdx = iEchoIdx = 0;
-	iEchoSeq = 0;
+	bPositionalCache = false;
+	bPulseIsGood = false;
 
 	pam = pa_mainloop_new();
 	pa_mainloop_api *api = pa_mainloop_get_api(pam);
 
 	pacContext = pa_context_new(api, "Mumble");
-	
+
 	pa_context_set_subscribe_callback(pacContext, subscribe_callback, this);
-	
+
 	pa_context_set_state_callback(pacContext, context_state_callback, this);
 	pa_context_connect(pacContext, NULL, PA_CONTEXT_NOAUTOSPAWN, NULL);
 
 	pade = api->defer_new(api, defer_event_callback, this);
 	api->defer_enable(pade, false);
-
-	int margin = 320;
-	jbJitter = jitter_buffer_init(margin);
-	jitter_buffer_ctl(jbJitter, JITTER_BUFFER_SET_MARGIN, &margin);
-	start(QThread::TimeCriticalPriority);
 }
 
 PulseAudioSystem::~PulseAudioSystem() {
 	pa_mainloop_quit(pam, 0);
 	wait();
-	jitter_buffer_destroy(jbJitter);
 	pa_context_disconnect(pacContext);
 	pa_context_unref(pacContext);
 	pa_mainloop_free(pam);
@@ -119,6 +126,9 @@ void PulseAudioSystem::wakeup() {
 }
 
 void PulseAudioSystem::run() {
+	// Make sure we don't exit too early.
+	qmWait.lock();
+	qmWait.unlock();
 	int rv;
 	pa_mainloop_run(pam, &rv);
 	if (rv != 0)
@@ -131,8 +141,7 @@ void PulseAudioSystem::defer_event_callback(pa_mainloop_api *a, pa_defer_event *
 }
 
 
-void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *evt) {
-	Q_ASSERT(pade == evt);
+void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *) {
 	api->defer_enable(pade, false);
 
 	if (! bSourceDone || ! bSinkDone || ! bServerDone)
@@ -158,7 +167,14 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *evt)
 				case PA_STREAM_TERMINATED: {
 						if (pasOutput)
 							pa_stream_unref(pasOutput);
-						pasOutput = pa_stream_new(pacContext, "Mumble Speakers", &ss, NULL);
+						pa_sample_spec pss = qhSpecMap.value(odev);
+						if (pss.rate == 0)
+							pss.rate = SAMPLE_RATE;
+						if ((pss.channels == 0) || (! g.s.doPositionalAudio()))
+							pss.channels = 1;
+						if ((pss.format != PA_SAMPLE_FLOAT32NE) && (pss.format != PA_SAMPLE_S16NE))
+							pss.format = PA_SAMPLE_FLOAT32NE;
+						pasOutput = pa_stream_new(pacContext, "Mumble Speakers", &pss, NULL);
 						pa_stream_set_state_callback(pasOutput, stream_callback, this);
 						pa_stream_set_write_callback(pasOutput, write_callback, this);
 					}
@@ -166,7 +182,9 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *evt)
 					do_start = true;
 					break;
 				case PA_STREAM_READY: {
-						if (g.s.iDXOutputDelay != iDelayCache) {
+						if (g.s.iOutputDelay != iDelayCache) {
+							do_stop = true;
+						} else if (g.s.doPositionalAudio() != bPositionalCache) {
 							do_stop = true;
 						} else if (odev != qsOutputCache) {
 							do_stop = true;
@@ -183,12 +201,15 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *evt)
 		} else if (do_start) {
 			qWarning("PulseAudio: Starting output: %s", qPrintable(odev));
 			pa_buffer_attr buff;
-			buff.maxlength = pao->iFrameSize * NBLOCKS * sizeof(short);
-			buff.tlength = g.s.iDXOutputDelay * pao->iFrameSize * sizeof(short);
+			const pa_sample_spec *pss = pa_stream_get_sample_spec(pasOutput);
+			const unsigned int iBlockLen = ((pao->iFrameSize * pss->rate) / SAMPLE_RATE) * pss->channels * ((pss->format == PA_SAMPLE_FLOAT32NE) ? sizeof(float) : sizeof(short));
+			buff.maxlength = iBlockLen * NBLOCKS;
+			buff.tlength = g.s.iOutputDelay * iBlockLen;
 			buff.prebuf = buff.tlength;
-			buff.minreq = pao->iFrameSize * sizeof(short);
+			buff.minreq = iBlockLen;
 
-			iDelayCache = g.s.iDXOutputDelay;
+			iDelayCache = g.s.iOutputDelay;
+			bPositionalCache = g.s.doPositionalAudio();
 			qsOutputCache = odev;
 
 			pa_stream_connect_playback(pasOutput, qPrintable(odev), &buff, PA_STREAM_INTERPOLATE_TIMING, NULL, NULL);
@@ -209,10 +230,16 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *evt)
 						if (pasInput)
 							pa_stream_unref(pasInput);
 
-						pasInput = pa_stream_new(pacContext, "Mumble Microphone", &ss, NULL);
+						pa_sample_spec pss = qhSpecMap.value(idev);
+						if (pss.rate == 0)
+							pss.rate = SAMPLE_RATE;
+						pss.channels = 1;
+						if ((pss.format != PA_SAMPLE_FLOAT32NE) && (pss.format != PA_SAMPLE_S16NE))
+							pss.format = PA_SAMPLE_FLOAT32NE;
+
+						pasInput = pa_stream_new(pacContext, "Mumble Microphone", &pss, NULL);
 						pa_stream_set_state_callback(pasInput, stream_callback, this);
 						pa_stream_set_read_callback(pasInput, read_callback, this);
-
 					}
 				case PA_STREAM_UNCONNECTED:
 					do_start = true;
@@ -233,8 +260,10 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *evt)
 		} else if (do_start) {
 			qWarning("PulseAudio: Starting input %s",qPrintable(idev));
 			pa_buffer_attr buff;
-			buff.maxlength = pai->iFrameSize * NBLOCKS * sizeof(short);
-			buff.fragsize = pai->iFrameSize * sizeof(short);
+			const pa_sample_spec *pss = pa_stream_get_sample_spec(pasInput);
+			const unsigned int iBlockLen = ((pai->iFrameSize * pss->rate) / SAMPLE_RATE) * pss->channels * ((pss->format == PA_SAMPLE_FLOAT32NE) ? sizeof(float) : sizeof(short));
+			buff.maxlength = iBlockLen * NBLOCKS;
+			buff.fragsize = iBlockLen;
 
 			qsInputCache = idev;
 
@@ -249,15 +278,21 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *evt)
 		bool do_stop = false;
 		bool do_start = false;
 
-		if ((! pai || ! g.s.bPulseAudioEcho) && (est == PA_STREAM_READY)) {
+		if ((! pai || ! g.s.doEcho()) && (est == PA_STREAM_READY)) {
 			do_stop = true;
-		} else if (pai && g.s.bPulseAudioEcho) {
+		} else if (pai && g.s.doEcho()) {
 			switch (est) {
 				case PA_STREAM_TERMINATED: {
 						if (pasSpeaker)
 							pa_stream_unref(pasSpeaker);
 
-						pasSpeaker = pa_stream_new(pacContext, "Mumble Speakers (Echo)", &ss, NULL);
+						pa_sample_spec pss = qhSpecMap.value(edev);
+						if (pss.rate == 0)
+							pss.rate = SAMPLE_RATE;
+						pss.channels = 1;
+						if ((pss.format != PA_SAMPLE_FLOAT32NE) && (pss.format != PA_SAMPLE_S16NE))
+							pss.format = PA_SAMPLE_FLOAT32NE;
+						pasSpeaker = pa_stream_new(pacContext, "Mumble Speakers (Echo)", &pss, NULL);
 						pa_stream_set_state_callback(pasSpeaker, stream_callback, this);
 						pa_stream_set_read_callback(pasSpeaker, read_callback, this);
 					}
@@ -280,8 +315,10 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *evt)
 		} else if (do_start) {
 			qWarning("PulseAudio: Starting echo: %s", qPrintable(edev));
 			pa_buffer_attr buff;
-			buff.maxlength = pai->iFrameSize * NBLOCKS * sizeof(short);
-			buff.fragsize = pai->iFrameSize * sizeof(short);
+			const pa_sample_spec *pss = pa_stream_get_sample_spec(pasSpeaker);
+			const unsigned int iBlockLen = ((pai->iFrameSize * pss->rate) / SAMPLE_RATE) * pss->channels * ((pss->format == PA_SAMPLE_FLOAT32NE) ? sizeof(float) : sizeof(short));
+			buff.maxlength = iBlockLen * NBLOCKS;
+			buff.fragsize = iBlockLen;
 
 			qsEchoCache = edev;
 
@@ -295,7 +332,7 @@ void PulseAudioSystem::context_state_callback(pa_context *c, void *userdata) {
 	pas->contextCallback(c);
 }
 
-void PulseAudioSystem::subscribe_callback(pa_context *c, pa_subscription_event_type t, unsigned int idx, void *userdata) {
+void PulseAudioSystem::subscribe_callback(pa_context *, pa_subscription_event_type, unsigned int, void *userdata) {
 	PulseAudioSystem *pas = reinterpret_cast<PulseAudioSystem *>(userdata);
 	qWarning("PulseAudio: Sinks or inputs changed (inserted or removed sound card)");
 	pas->query();
@@ -309,10 +346,12 @@ void PulseAudioSystem::sink_callback(pa_context *, const pa_sink_info *i, int eo
 		return;
 	}
 
-	pas->qhIndexMap.insert(QLatin1String(i->name), i->index);
+	const QString name = QLatin1String(i->name);
 
-	pas->qhOutput.insert(QLatin1String(i->name), QLatin1String(i->description));
-	pas->qhEchoMap.insert(QLatin1String(i->name), QLatin1String(i->monitor_source_name));
+	pas->qhIndexMap.insert(name, i->index);
+	pas->qhSpecMap.insert(name, i->sample_spec);
+	pas->qhOutput.insert(name, QLatin1String(i->description));
+	pas->qhEchoMap.insert(name, QLatin1String(i->monitor_source_name));
 }
 
 void PulseAudioSystem::source_callback(pa_context *, const pa_source_info *i, int eol, void *userdata) {
@@ -323,7 +362,10 @@ void PulseAudioSystem::source_callback(pa_context *, const pa_source_info *i, in
 		return;
 	}
 
-	pas->qhIndexMap.insert(QLatin1String(i->name), i->index);
+	const QString name = QLatin1String(i->name);
+
+	pas->qhIndexMap.insert(name, i->index);
+	pas->qhSpecMap.insert(name, i->sample_spec);
 
 	if (i->monitor_of_sink == PA_INVALID_INDEX)
 		pas->qhInput.insert(QLatin1String(i->name), QLatin1String(i->description));
@@ -357,9 +399,7 @@ void PulseAudioSystem::read_callback(pa_stream *s, size_t bytes, void *userdata)
 	size_t length = bytes;
 	const void *data;
 	pa_stream_peek(s, &data, &length);
-	const short *buffer = reinterpret_cast<const short *>(data);
-	int samples = length / 2;
-	
+
 	AudioInputPtr ai = g.ai;
 	PulseAudioInput *pai = dynamic_cast<PulseAudioInput *>(ai.get());
 	if (! pai) {
@@ -368,55 +408,32 @@ void PulseAudioSystem::read_callback(pa_stream *s, size_t bytes, void *userdata)
 		return;
 	}
 
-	// PulseAudio supports setting fragments. But they're only avisory,
-	// so we have to copy data like mad.
-	if (! pas->psInput)
-		pas->psInput = new short[pai->iFrameSize];
-	if (! pas->psEcho)
-		pas->psEcho = new short[pai->iFrameSize];
+	const pa_sample_spec *pss = pa_stream_get_sample_spec(s);
 
 	if (s == pas->pasInput) {
-		while (samples > 0) {
-			int ncopy = MIN(pai->iFrameSize - pas->iInputIdx, samples);
-			memcpy(pas->psInput + pas->iInputIdx, buffer, ncopy * sizeof(short));
-			pas->iInputIdx += ncopy;
-			buffer += ncopy;
-			samples -= ncopy;
-			if (pas->iInputIdx == pai->iFrameSize) {
-				pas->iInputIdx = 0;
-				memcpy(pai->psMic, pas->psInput, pai->iFrameSize * sizeof(short));
-
-				if (g.s.bPulseAudioEcho) {
-					JitterBufferPacket jbp;
-					jbp.data = reinterpret_cast<char *>(pai->psSpeaker);
-					jbp.len = pai->iFrameSize * sizeof(short);
-					spx_int32_t startofs = 0;
-					jitter_buffer_get(pas->jbJitter, &jbp, pai->iFrameSize, &startofs);
-					jitter_buffer_update_delay(pas->jbJitter, &jbp, NULL);
-					jitter_buffer_tick(pas->jbJitter);
-				}
-				pai->encodeAudioFrame();
-			}
+		if (!pa_sample_spec_equal(pss, &pai->pssMic)) {
+			pai->pssMic = *pss;
+			pai->iMicFreq = pss->rate;
+			pai->iMicChannels = pss->channels;
+			if (pss->format == PA_SAMPLE_FLOAT32NE)
+				pai->eMicFormat = AudioInput::SampleFloat;
+			else
+				pai->eMicFormat = AudioInput::SampleShort;
+			pai->initializeMixer();
 		}
+		pai->addMic(data, length / pai->iMicSampleSize);
 	} else if (s == pas->pasSpeaker) {
-		while (samples > 0) {
-			int ncopy = MIN(pai->iFrameSize - pas->iEchoIdx, samples);
-			memcpy(pas->psEcho + pas->iEchoIdx, buffer, ncopy * sizeof(short));
-			pas->iEchoIdx += ncopy;
-			buffer += ncopy;
-			samples -= ncopy;
-			if (pas->iEchoIdx == pai->iFrameSize) {
-				pas->iEchoIdx = 0;
-
-				JitterBufferPacket jbp;
-				jbp.data = reinterpret_cast<char *>(pas->psEcho);
-				jbp.len = pai->iFrameSize * sizeof(short);
-				jbp.span = pai->iFrameSize;
-				jbp.timestamp = pai->iFrameSize * (++(pas->iEchoSeq));
-
-				jitter_buffer_put(pas->jbJitter, &jbp);
-			}
+		if (!pa_sample_spec_equal(pss, &pai->pssEcho)) {
+			pai->pssEcho = *pss;
+			pai->iEchoFreq = pss->rate;
+			pai->iEchoChannels = pss->channels;
+			if (pss->format == PA_SAMPLE_FLOAT32NE)
+				pai->eEchoFormat = AudioInput::SampleFloat;
+			else
+				pai->eEchoFormat = AudioInput::SampleShort;
+			pai->initializeMixer();
 		}
+		pai->addEcho(data, length / pai->iEchoSampleSize);
 	}
 
 	pa_stream_drop(s);
@@ -428,25 +445,82 @@ void PulseAudioSystem::write_callback(pa_stream *s, size_t bytes, void *userdata
 
 	AudioOutputPtr ao = g.ao;
 	PulseAudioOutput *pao = dynamic_cast<PulseAudioOutput *>(ao.get());
-	
-	int samples = bytes / 2;
-	
+
+	int samples = bytes / sizeof(float);
+	float buffer[samples];
+
 	if (! pao) {
 		// Transitioning, but most likely transitions back, so just zero.
-		short zero[samples];
-		memset(zero, 0, samples*sizeof(short));
-		pa_stream_write(s, zero, samples * sizeof(short), NULL, 0, PA_SEEK_RELATIVE);
+		memset(buffer, 0, bytes);
+		pa_stream_write(s, buffer, bytes, NULL, 0, PA_SEEK_RELATIVE);
 		pas->wakeup();
 		return;
 	}
 
-	short buffer[pao->iFrameSize] __attribute__((aligned(16)));
-
-	while (samples >= 0) {
-		samples -= pao->iFrameSize;
-		pao->mixAudio(buffer);
-		pa_stream_write(s, buffer, pao->iFrameSize * sizeof(short), NULL, 0, PA_SEEK_RELATIVE);
+	const pa_sample_spec *pss = pa_stream_get_sample_spec(s);
+	const pa_channel_map *pcm = pa_stream_get_channel_map(pas->pasOutput);
+	if (!pa_sample_spec_equal(pss, &pao->pss) || !pa_channel_map_equal(pcm, &pao->pcm)) {
+		pao->pss = *pss;
+		pao->pcm = *pcm;
+		if (pss->format == PA_SAMPLE_FLOAT32NE)
+			pao->eSampleFormat = AudioOutput::SampleFloat;
+		else
+			pao->eSampleFormat = AudioOutput::SampleShort;
+		pao->iMixerFreq = pss->rate;
+		pao->iChannels = pss->channels;
+		unsigned int chanmasks[pss->channels];
+		for (int i=0;i<pss->channels;++i) {
+			unsigned int cm = 0;
+			switch (pcm->map[i]) {
+				case PA_CHANNEL_POSITION_LEFT:
+					cm = SPEAKER_FRONT_LEFT;
+					break;
+				case PA_CHANNEL_POSITION_RIGHT:
+					cm = SPEAKER_FRONT_RIGHT;
+					break;
+				case PA_CHANNEL_POSITION_CENTER:
+					cm = SPEAKER_FRONT_CENTER;
+					break;
+				case PA_CHANNEL_POSITION_REAR_LEFT:
+					cm = SPEAKER_BACK_LEFT;
+					break;
+				case PA_CHANNEL_POSITION_REAR_RIGHT:
+					cm = SPEAKER_BACK_RIGHT;
+					break;
+				case PA_CHANNEL_POSITION_REAR_CENTER:
+					cm = SPEAKER_BACK_CENTER;
+					break;
+				case PA_CHANNEL_POSITION_LFE:
+					cm = SPEAKER_LOW_FREQUENCY;
+					break;
+				case PA_CHANNEL_POSITION_SIDE_LEFT:
+					cm = SPEAKER_SIDE_LEFT;
+					break;
+				case PA_CHANNEL_POSITION_SIDE_RIGHT:
+					cm = SPEAKER_SIDE_RIGHT;
+					break;
+				case PA_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER:
+					cm = SPEAKER_FRONT_LEFT_OF_CENTER;
+					break;
+				case PA_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER:
+					cm = SPEAKER_FRONT_RIGHT_OF_CENTER;
+					break;
+				default:
+					cm = 0;
+					break;
+			}
+			chanmasks[i] = cm;
+		}
+		pao->initializeMixer(chanmasks);
 	}
+
+
+	const unsigned int iSampleSize = pao->iSampleSize;
+	samples = bytes / iSampleSize;
+
+	if (! pao->mix(buffer, samples))
+		memset(buffer, 0, samples * iSampleSize);
+	pa_stream_write(s, buffer, iSampleSize * samples, NULL, 0, PA_SEEK_RELATIVE);
 }
 
 void PulseAudioSystem::query() {
@@ -466,6 +540,7 @@ void PulseAudioSystem::contextCallback(pa_context *c) {
 	Q_ASSERT(c == pacContext);
 	switch (pa_context_get_state(c)) {
 		case PA_CONTEXT_READY:
+			bPulseIsGood = true;
 			pa_operation_unref(pa_context_subscribe(pacContext, PA_SUBSCRIPTION_MASK_SOURCE, NULL, this));
 			pa_operation_unref(pa_context_subscribe(pacContext, PA_SUBSCRIPTION_MASK_SINK, NULL, this));
 			query();
@@ -477,11 +552,12 @@ void PulseAudioSystem::contextCallback(pa_context *c) {
 			qWarning("PulseAudio: Connection failure: %s", pa_strerror(pa_context_errno(c)));
 			break;
 		default:
-			break;
+			return;
 	}
+	qwcWait.wakeAll();
 }
 
-PulseAudioInputRegistrar::PulseAudioInputRegistrar() : AudioInputRegistrar(QLatin1String("PulseAudio")) {
+PulseAudioInputRegistrar::PulseAudioInputRegistrar() : AudioInputRegistrar(QLatin1String("PulseAudio"), 10) {
 }
 
 AudioInput *PulseAudioInputRegistrar::create() {
@@ -506,11 +582,15 @@ const QList<audioDevice> PulseAudioInputRegistrar::getDeviceChoices() {
 	return qlReturn;
 }
 
-void PulseAudioInputRegistrar::setDeviceChoice(const QVariant &choice) {
-	g.s.qsPulseAudioInput = choice.toString();
+void PulseAudioInputRegistrar::setDeviceChoice(const QVariant &choice, Settings &s) {
+	s.qsPulseAudioInput = choice.toString();
 }
 
-PulseAudioOutputRegistrar::PulseAudioOutputRegistrar() : AudioOutputRegistrar(QLatin1String("PulseAudio")) {
+bool PulseAudioInputRegistrar::canEcho(const QString &osys) {
+	return (osys == name);
+}
+
+PulseAudioOutputRegistrar::PulseAudioOutputRegistrar() : AudioOutputRegistrar(QLatin1String("PulseAudio"), 10) {
 }
 
 AudioOutput *PulseAudioOutputRegistrar::create() {
@@ -535,78 +615,14 @@ const QList<audioDevice> PulseAudioOutputRegistrar::getDeviceChoices() {
 	return qlReturn;
 }
 
-void PulseAudioOutputRegistrar::setDeviceChoice(const QVariant &choice) {
-	g.s.qsPulseAudioOutput = choice.toString();
-}
-
-static ConfigWidget *PulseAudioConfigDialogNew(Settings &st) {
-	return new PulseAudioConfig(st);
-}
-
-static ConfigRegistrar registrar(22, PulseAudioConfigDialogNew);
-
-PulseAudioConfig::PulseAudioConfig(Settings &st) : ConfigWidget(st) {
-	setupUi(this);
-
-	QStringList qlOutputDevs = pasys->qhOutput.keys();
-	qSort(qlOutputDevs);
-	QStringList qlInputDevs = pasys->qhInput.keys();
-	qSort(qlInputDevs);
-
-	foreach(QString dev, qlInputDevs) {
-		qcbInputDevice->addItem(pasys->qhInput.value(dev), dev);
-	}
-
-	foreach(QString dev, qlOutputDevs) {
-		qcbOutputDevice->addItem(pasys->qhOutput.value(dev), dev);
-	}
-}
-
-QString PulseAudioConfig::title() const {
-	return tr("PulseAudio");
-}
-
-QIcon PulseAudioConfig::icon() const {
-	return QIcon(QLatin1String("skin:config_dsound.png"));
-}
-
-void PulseAudioConfig::save() const {
-	s.iDXOutputDelay = qsOutputDelay->value();
-	s.qsPulseAudioInput =  qcbInputDevice->itemData(qcbInputDevice->currentIndex()).toString();
-	s.qsPulseAudioOutput =  qcbOutputDevice->itemData(qcbOutputDevice->currentIndex()).toString();
-	s.bPulseAudioEcho = qcbEcho->isChecked();
-}
-
-void PulseAudioConfig::load(const Settings &r) {
-	for (int i=0;i<qcbInputDevice->count();i++) {
-		if (qcbInputDevice->itemData(i).toString() == r.qsPulseAudioInput) {
-			loadComboBox(qcbInputDevice, i);
-			break;
-		}
-	}
-
-	for (int i=0;i<qcbOutputDevice->count();i++) {
-		if (qcbOutputDevice->itemData(i).toString() == r.qsPulseAudioOutput) {
-			loadComboBox(qcbOutputDevice, i);
-			break;
-		}
-	}
-	loadSlider(qsOutputDelay, r.iDXOutputDelay);
-	loadCheckBox(qcbEcho, r.bPulseAudioEcho);
-}
-
-bool PulseAudioConfig::expert(bool b) {
-	qcbOutput->setVisible(b);
-	return true;
-}
-
-void PulseAudioConfig::on_qsOutputDelay_valueChanged(int v) {
-	qlOutputDelay->setText(tr("%1ms").arg(v*20));
+void PulseAudioOutputRegistrar::setDeviceChoice(const QVariant &choice, Settings &s) {
+	s.qsPulseAudioOutput = choice.toString();
 }
 
 PulseAudioInput::PulseAudioInput() {
+	pssMic.rate = 0;
+	pssEcho.rate = 0;
 	bRunning = true;
-	bHasSpeaker = g.s.bPulseAudioEcho;
 	if (pasys)
 		pasys->wakeup();
 };
@@ -618,6 +634,8 @@ PulseAudioInput::~PulseAudioInput() {
 }
 
 PulseAudioOutput::PulseAudioOutput() {
+	pss.rate = 0;
+	pcm.channels = 0;
 	bRunning = true;
 	if (pasys)
 		pasys->wakeup();
