@@ -45,7 +45,7 @@ struct Settings {
 	enum VADSource { Amplitude, SignalToNoise };
 	enum LoopMode { None, Local, Server };
 	enum OverlayShow { Nothing, Talking, All };
-	enum Audio3D { No3D, Panning, Light, Full };
+	enum ChannelExpand { NoChannels, ChannelsWithPlayers, AllChannels };
 	AudioTransmit atTransmit;
 	quint64 uiDoublePush;
 
@@ -62,25 +62,26 @@ struct Settings {
 	VADSource vsVAD;
 	float fVADmin, fVADmax;
 	int iFramesPerPacket;
-	bool bExpandAll;
 	QString qsAudioInput, qsAudioOutput;
 	float fVolume;
+	int iOutputDelay;
 
-	int iDXOutputDelay;
-
-	QString qsFestival, qsFestivalPattern, qsFestivalSearch, qsFestivalReplace;
 	QString qsALSAInput, qsALSAOutput;
 	QString qsPulseAudioInput, qsPulseAudioOutput;
-	bool bPulseAudioEcho;
 	QString qsOSSInput, qsOSSOutput;
 	int iPortAudioInput, iPortAudioOutput;
 	QString qsASIOclass;
 	QList<QVariant> qlASIOmic;
 	QList<QVariant> qlASIOspeaker;
-	Audio3D a3dModel;
+
+	QString qsWASAPIInput, qsWASAPIOutput;
 
 	QByteArray qbaDXInput, qbaDXOutput;
-	float fDXMinDistance, fDXMaxDistance, fDXRollOff;
+
+	bool bEcho;
+	bool bPositionalAudio;
+	bool bPositionalHeadphone;
+	float fAudioMinDistance, fAudioMaxDistance, fAudioRollOff, fAudioBloom;
 
 	bool bOverlayEnable;
 	bool bOverlayUserTextures;
@@ -106,9 +107,10 @@ struct Settings {
 	QString qsLanguage;
 	QString qsStyle;
 	QString qsSkin;
-	bool bHorizontal;
 	bool bFirstTime;
 	QByteArray qbaMainWindowGeometry, qbaMainWindowState, qbaSplitterState, qbaHeaderState;
+	ChannelExpand ceExpand;
+	bool bMinimalView;
 
 	QString qsUsername;
 	int iServerRow;
@@ -123,10 +125,12 @@ struct Settings {
 	unsigned short usProxyPort;
 
 	// Nonsaved
-	bool bLocalDeafen;
 	LoopMode lmLoopMode;
-	double dPacketLoss;
-	double dMaxPacketDelay;
+	float dPacketLoss;
+	float dMaxPacketDelay;
+
+	bool doEcho() const;
+	bool doPositionalAudio() const;
 
 	Settings();
 	void load();
@@ -134,5 +138,5 @@ struct Settings {
 };
 
 #else
-class Settings;
+struct Settings;
 #endif
