@@ -71,13 +71,31 @@ int main(int argc, char **argv) {
 	a.setOrganizationDomain(QLatin1String("mumble.sourceforge.net"));
 	a.setQuitOnLastWindowClosed(false);
 
+#ifdef Q_OS_MAC
+	// Avoid auto-loading system Qt plugins. This could potentially make
+	// us load two different Qt libraries into our process. Not good.
+	a.setLibraryPaths(QStringList());
+#endif
+
 	Global::g_global_struct = new Global();
 
-#ifndef Q_OS_MAC
+#ifdef USE_DBUS
 #ifdef Q_OS_WIN
 	// By default, windbus expects the path to dbus-daemon to be in PATH, and the path
 	// should contain bin\\, and the path to the config is hardcoded as ..\etc
-	_putenv(qPrintable(QString::fromLatin1("PATH=%1\\bin;%1;%2").arg(QDir::toNativeSeparators(a.applicationDirPath())).arg(QLatin1String(getenv("PATH")))));
+
+	{
+		size_t reqSize;
+		_wgetenv_s(&reqSize, NULL, 0, L"PATH");
+		if (reqSize > 0) {
+			STACKVAR(wchar_t, buff, reqSize+1);
+			_wgetenv_s(&reqSize, buff, reqSize, L"PATH");
+			QString path = QString::fromLatin1("%1\\bin;%1;%2").arg(QDir::toNativeSeparators(a.applicationDirPath())).arg(QString::fromWCharArray(buff));
+			STACKVAR(wchar_t, buffout, path.length() + 1);
+			path.toWCharArray(buffout);
+			_wputenv_s(L"PATH", buffout);
+		}
+	}
 #endif
 
 	QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble"), QLatin1String("/"));
@@ -105,7 +123,6 @@ int main(int argc, char **argv) {
 		a.setStyle(g.s.qsStyle);
 	}
 
-
 	if (! g.s.qsSkin.isEmpty()) {
 		QFile file(g.s.qsSkin);
 		file.open(QFile::ReadOnly);
@@ -133,19 +150,6 @@ int main(int argc, char **argv) {
 	qttranslator.load(QLatin1String("translation:qt_") + locale);
 	a.installTranslator(&qttranslator);
 
-
-#ifndef Q_OS_MAC
-	// Set application icon except on MacOSX, where the window-icon
-	// shown in the title-bar usually serves as a draggable version of the
-	// current open document (i.e. you can copy the open document anywhere
-	// simply by dragging this icon).
-	QIcon icon;
-	icon.addFile(QLatin1String("skin:mumble.16x16.png"));
-	icon.addFile(QLatin1String("skin:mumble.32x32.png"));
-	icon.addFile(QLatin1String("skin:mumble.64x64.png"));
-	a.setWindowIcon(icon);
-#endif
-
 	// Initialize proxy settings
 	NetworkConfig::SetupProxy();
 
@@ -166,10 +170,10 @@ int main(int argc, char **argv) {
 	g.mw=new MainWindow(NULL);
 	g.mw->show();
 
-#ifndef Q_OS_MAC
+#ifdef USE_DBUS
 	new MumbleDBus(g.mw);
-	QDBusConnection::sessionBus().registerObject("/", g.mw);
-	QDBusConnection::sessionBus().registerService("net.sourceforge.mumble.mumble");
+	QDBusConnection::sessionBus().registerObject(QLatin1String("/"), g.mw);
+	QDBusConnection::sessionBus().registerService(QLatin1String("net.sourceforge.mumble.mumble"));
 #endif
 
 	g.l->log(Log::Information, MainWindow::tr("Welcome to Mumble."));
@@ -180,17 +184,20 @@ int main(int argc, char **argv) {
 
 	// And the start the last chosen audio system.
 	g.ai = AudioInputRegistrar::newFromChoice();
-	g.ai->start(QThread::HighestPriority);
+	if (g.ai)
+		g.ai->start(QThread::HighestPriority);
 	g.ao = AudioOutputRegistrar::newFromChoice();
-	g.ao->start(QThread::HighPriority);
+	if (g.ao)
+		g.ao->start(QThread::HighPriority);
 
 	a.setQuitOnLastWindowClosed(false);
 
 	if (g.s.bFirstTime) {
 		g.s.bFirstTime = false;
 		if (QMessageBox::question(g.mw, MainWindow::tr("Mumble"), MainWindow::tr("This is the first time you're starting Mumble.<br />Would you like to go through the Audio Wizard to configure your soundcard?"), QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes) == QMessageBox::Yes) {
-			AudioWizard aw;
-			aw.exec();
+			AudioWizard *aw = new AudioWizard(g.mw);
+			aw->exec();
+			delete aw;
 		}
 	}
 
