@@ -49,7 +49,7 @@ MetaParams::MetaParams() {
 	qsDBDriver = "QSQLITE";
 	qsLogfile = "murmur.log";
 	qhaBind = QHostAddress(QHostAddress::Any);
-	
+
 	iLogDays = 31;
 
 	iBanTries = 10;
@@ -59,14 +59,53 @@ MetaParams::MetaParams() {
 
 void MetaParams::read(QString fname) {
 	if (fname.isEmpty()) {
-		fname = "murmur.ini";
+		QStringList datapaths;
+
+#if defined(Q_OS_WIN)
+		size_t reqSize;
+		_wgetenv_s(&reqSize, NULL, 0, L"APPDATA");
+		if (reqSize > 0) {
+			STACKVAR(wchar_t, buff, reqSize+1);
+			_wgetenv_s(&reqSize, buff, reqSize, L"APPDATA");
+
+			QDir appdir = QDir(QDir::fromNativeSeparators(QString::fromWCharArray(buff)));
+
+			datapaths << appdir.absolutePath() + QLatin1String("/Mumble");
+		}
+#elif defined(Q_OS_MAC)
+		datapaths << QDir::homePath() + QLatin1String("/Library/Preferences/Mumble/");
+#else
+		datapaths << QDir::homePath() + QLatin1String("/.config/Mumble");
+#endif
+		datapaths << QDir::homePath();
+		datapaths << QDir::currentPath();
+		datapaths << QCoreApplication::instance()->applicationDirPath();
+
+		foreach(const QString &p, datapaths) {
+			QFileInfo fi(p, "murmur.ini");
+			if (fi.exists() && fi.isReadable()) {
+				qdBasePath = QDir(p);
+				fname = fi.absoluteFilePath();
+				break;
+			}
+		}
+		if (fname.isEmpty()) {
+			QDir::root().mkpath(qdBasePath.absolutePath());
+			qdBasePath = QDir(datapaths.at(0));
+			fname = qdBasePath.absolutePath() + QLatin1String("/murmur.ini");
+		}
 	} else {
-		if (! QFile(fname).exists())
+		QFile f(fname);
+		if (! f.open(QIODevice::ReadOnly)) {
 			qFatal("Specified ini file %s could not be opened", qPrintable(fname));
+		}
+		qdBasePath = QFileInfo(f).absoluteDir();
+		f.close();
 	}
+	QDir::setCurrent(qdBasePath.absolutePath());
 	QSettings qs(fname, QSettings::IniFormat);
 
-	qWarning("Initializing settings from %s", qPrintable(qs.fileName()));
+	qWarning("Initializing settings from %s (basepath %s)", qPrintable(qs.fileName()), qPrintable(qdBasePath.absolutePath()));
 
 	QString qsHost = qs.value("host", QString()).toString();
 	if (! qsHost.isEmpty()) {
@@ -102,7 +141,7 @@ void MetaParams::read(QString fname) {
 	qsDBHostName = qs.value("dbHost", qsDBHostName).toString();
 	qsDBPrefix = qs.value("dbPrefix", qsDBPrefix).toString();
 	iDBPort = qs.value("dbPort", iDBPort).toInt();
-	
+
 	iLogDays = qs.value("logdays", iLogDays).toInt();
 
 	qsDBus = qs.value("dbus", qsDBus).toString();
@@ -182,7 +221,7 @@ void MetaParams::read(QString fname) {
 			qFatal("Certificate specified, but failed to load.");
 		}
 	}
-	
+
 	qmConfig.clear();
 	qmConfig.insert(QLatin1String("host"),qhaBind.toString());
 	qmConfig.insert(QLatin1String("password"),qsPassword);
