@@ -28,6 +28,8 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "murmur_pch.h"
+
 #include "Player.h"
 #include "Channel.h"
 #include "ACL.h"
@@ -129,7 +131,7 @@ Server::Server(int snum, QObject *p) : QThread(p) {
 
 	qtTimeout = new QTimer(this);
 	connect(qtTimeout, SIGNAL(timeout()), this, SLOT(checkTimeout()));
-	qtTimeout->start(5500);
+	qtTimeout->start(15500);
 
 	getBans();
 	readChannels();
@@ -202,6 +204,32 @@ void Server::readParams() {
 	qurlRegWeb = QUrl(getConf("registerurl", qurlRegWeb.toString()).toString());
 }
 
+void Server::setLiveConf(const QString &key, const QString &value) {
+	QString v = value.trimmed().isEmpty() ? QString() : value;
+	int i = v.toInt();
+	if (key == "password")
+		qsPassword = !v.isNull() ? v : Meta::mp.qsPassword;
+	else if (key == "timeout")
+		iTimeout = i ? i : Meta::mp.iTimeout;
+	else if (key == "bandwidth")
+		iMaxBandwidth = i ? i : Meta::mp.iMaxBandwidth;
+	else if (key == "users")
+		iMaxUsers = i ? i : Meta::mp.iMaxUsers;
+	else if (key == "defaultchannel")
+		iDefaultChan = i ? i : Meta::mp.iDefaultChan;
+	else if (key == "welcometext")
+		qsWelcomeText = !v.isNull() ? v : Meta::mp.qsWelcomeText;
+	else if (key == "registername")
+		qsRegName = !v.isNull() ? v : Meta::mp.qsRegName;
+	else if (key == "registerpassword")
+		qsRegPassword = !v.isNull() ? v : Meta::mp.qsRegPassword;
+	else if (key == "registerhostname")
+		qsRegHost = !v.isNull() ? v : Meta::mp.qsRegHost;
+	else if (key == "registerurl")
+		qurlRegWeb = !v.isNull() ? v : Meta::mp.qurlRegWeb;
+}
+
+
 BandwidthRecord::BandwidthRecord() {
 	iRecNum = 0;
 	iSum = 0;
@@ -343,7 +371,7 @@ bool Server::checkDecrypt(User *u, const char *encrypted, char *plain, unsigned
 
 void Server::sendMessage(User *u, const char *data, int len, QByteArray &cache) {
 	if ((u->saiUdpAddress.sin_port != 0) && u->csCrypt.isValid()) {
-		char buffer[len+4];
+		STACKVAR(char, buffer, len+4);
 		u->csCrypt.encrypt(reinterpret_cast<const unsigned char *>(data), reinterpret_cast<unsigned char *>(buffer), len);
 		::sendto(sUdpSocket, buffer, len+4, 0, reinterpret_cast<struct sockaddr *>(& u->saiUdpAddress), sizeof(u->saiUdpAddress));
 	} else {
