@@ -41,28 +41,31 @@
 #define SQLDO(x) ServerDB::exec(query, QLatin1String(x), false)
 #define SQLPREP(x) ServerDB::prepare(query, QLatin1String(x))
 #define SQLEXEC() ServerDB::exec(query)
+#define SOFTEXEC() ServerDB::exec(query, QString(), false)
 
 class TransactionHolder {
-	protected:
-		bool bAbort;
 	public:
+		QSqlQuery *qsqQuery;
 		TransactionHolder() {
-			ServerDB::db.transaction();
+			ServerDB::db->transaction();
+			qsqQuery = new QSqlQuery();
 		}
 
 		~TransactionHolder() {
-			ServerDB::db.commit();
+			qsqQuery->clear();
+			delete qsqQuery;
+			ServerDB::db->commit();
 		}
 };
 
-QSqlDatabase ServerDB::db;
+QSqlDatabase *ServerDB::db;
 Timer ServerDB::tLogClean;
 
 ServerDB::ServerDB() {
 	if (! QSqlDatabase::isDriverAvailable(Meta::mp.qsDBDriver)) {
-		qFatal("Database driver %s not available", qPrintable(Meta::mp.qsDBDriver));
+		qFatal("ServerDB: Database driver %s not available", qPrintable(Meta::mp.qsDBDriver));
 	}
-	db = QSqlDatabase::addDatabase(Meta::mp.qsDBDriver);
+	db = new QSqlDatabase(QSqlDatabase::addDatabase(Meta::mp.qsDBDriver));
 	QStringList datapaths;
 	int i;
 
@@ -70,19 +73,20 @@ ServerDB::ServerDB() {
 
 	if (Meta::mp.qsDBDriver == "QSQLITE") {
 		if (! Meta::mp.qsDatabase.isEmpty()) {
-			db.setDatabaseName(Meta::mp.qsDatabase);
-			found = db.open();
+			db->setDatabaseName(Meta::mp.qsDatabase);
+			found = db->open();
 		} else {
-			datapaths << QCoreApplication::instance()->applicationDirPath();
+			datapaths << Meta::mp.qdBasePath.absolutePath();
 			datapaths << QDir::currentPath();
+			datapaths << QCoreApplication::instance()->applicationDirPath();
 			datapaths << QDir::homePath();
 
 			for (i = 0; (i < datapaths.size()) && ! found; i++) {
 				if (!datapaths[i].isEmpty()) {
 					QFile f(datapaths[i] + "/murmur.sqlite");
 					if (f.exists()) {
-						db.setDatabaseName(f.fileName());
-						found = db.open();
+						db->setDatabaseName(f.fileName());
+						found = db->open();
 					}
 				}
 			}
@@ -91,27 +95,29 @@ ServerDB::ServerDB() {
 				for (i = 0; (i < datapaths.size()) && ! found; i++) {
 					if (!datapaths[i].isEmpty()) {
 						QFile f(datapaths[i] + "/murmur.sqlite");
-						db.setDatabaseName(f.fileName());
-						found = db.open();
+						db->setDatabaseName(f.fileName());
+						found = db->open();
 					}
 				}
 			}
 		}
 		if (found) {
-			QFileInfo fi(db.databaseName());
-			qWarning("Openend SQLite database %s", qPrintable(fi.absoluteFilePath()));
+			QFileInfo fi(db->databaseName());
+			qWarning("ServerDB: Openend SQLite database %s", qPrintable(fi.absoluteFilePath()));
+			if (! fi.isWritable())
+				qFatal("ServerDB: Database is not writeable");
 		}
 	} else {
-		db.setDatabaseName(Meta::mp.qsDatabase);
-		db.setHostName(Meta::mp.qsDBHostName);
-		db.setPort(Meta::mp.iDBPort);
-		db.setUserName(Meta::mp.qsDBUserName);
-		db.setPassword(Meta::mp.qsDBPassword);
-		found = db.open();
+		db->setDatabaseName(Meta::mp.qsDatabase);
+		db->setHostName(Meta::mp.qsDBHostName);
+		db->setPort(Meta::mp.iDBPort);
+		db->setUserName(Meta::mp.qsDBUserName);
+		db->setPassword(Meta::mp.qsDBPassword);
+		found = db->open();
 	}
 
 	if (! found) {
-		QSqlError e = db.lastError();
+		QSqlError e = db->lastError();
 		qFatal("ServerDB: Failed initialization: %s",qPrintable(e.text()));
 	}
 
@@ -301,14 +307,17 @@ ServerDB::ServerDB() {
 			SQLDO("ALTER TABLE %1slog CONVERT TO CHARACTER SET utf8 COLLATE utf8_unicode_ci");
 		}
 	}
+	query.clear();
 }
 
 ServerDB::~ServerDB() {
-	db.close();
+	db->close();
+	delete db;
+	db = NULL;
 }
 
 bool ServerDB::prepare(QSqlQuery &query, const QString &str, bool fatal) {
-	if (! db.isValid()) {
+	if (! db->isValid()) {
 		qWarning("SQL [%s] rejected: Database is gone", qPrintable(str));
 		return false;
 	}
@@ -321,9 +330,9 @@ bool ServerDB::prepare(QSqlQuery &query, const QString &str, bool fatal) {
 	if (query.prepare(q)) {
 		return true;
 	} else {
-		db.close();
-		if (! db.open()) {
-			qFatal("Lost connection to SQL Database: Reconnect: %s", qPrintable(db.lastError().text()));
+		db->close();
+		if (! db->open()) {
+			qFatal("Lost connection to SQL Database: Reconnect: %s", qPrintable(db->lastError().text()));
 		}
 		query = QSqlQuery();
 		if (query.prepare(q)) {
@@ -332,7 +341,7 @@ bool ServerDB::prepare(QSqlQuery &query, const QString &str, bool fatal) {
 		}
 
 		if (fatal) {
-			db = QSqlDatabase();
+			*db = QSqlDatabase();
 			qFatal("SQL Prepare Error [%s]: %s", qPrintable(q), qPrintable(query.lastError().text()));
 		} else
 			qDebug("SQL Prepare Error [%s]: %s", qPrintable(q), qPrintable(query.lastError().text()));
@@ -348,7 +357,7 @@ bool ServerDB::exec(QSqlQuery &query, const QString &str, bool fatal) {
 	} else {
 
 		if (fatal) {
-			db = QSqlDatabase();
+			*db = QSqlDatabase();
 			qFatal("SQL Error [%s]: %s", qPrintable(query.lastQuery()), qPrintable(query.lastError().text()));
 		} else
 			qDebug("SQL Error [%s]: %s", qPrintable(query.lastQuery()), qPrintable(query.lastError().text()));
@@ -433,6 +442,7 @@ void Server::initialize() {
 			SQLEXEC();
 		}
 	}
+	query.clear();
 }
 
 int Server::registerPlayer(const QString &name) {
@@ -452,7 +462,7 @@ int Server::registerPlayer(const QString &name) {
 
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 
 	SQLPREP("SELECT MAX(player_id)+1 AS id FROM %1players WHERE server_id=? AND player_id < 1000000000");
 	query.addBindValue(iServerNum);
@@ -488,7 +498,7 @@ bool Server::unregisterPlayer(int id) {
 
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("DELETE FROM %1players WHERE server_id = ? AND player_id = ?");
 	query.addBindValue(iServerNum);
 	query.addBindValue(id);
@@ -504,7 +514,7 @@ QMap<int, QPair<QString, QString> > Server::getRegisteredPlayers(const QString &
 
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	if (filter.isEmpty()) {
 		SQLPREP("SELECT player_id, name, email FROM %1players WHERE server_id = ?");
 		query.addBindValue(iServerNum);
@@ -531,7 +541,7 @@ bool Server::getRegistration(int id, QString &name, QString &email) {
 
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("SELECT name, email FROM %1players WHERE server_id = ? AND player_id = ?");
 	query.addBindValue(iServerNum);
 	query.addBindValue(id);
@@ -552,7 +562,7 @@ int Server::authenticate(QString &name, const QString &pw) {
 	if (res != -2) {
 		if (res != -1) {
 			TransactionHolder th;
-			QSqlQuery query;
+			QSqlQuery &query = *th.qsqQuery;
 
 			int lchan=readLastChannel(res);
 
@@ -567,7 +577,7 @@ int Server::authenticate(QString &name, const QString &pw) {
 	}
 
 	TransactionHolder th;
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 
 	SQLPREP("SELECT player_id,name,pw FROM %1players WHERE server_id = ? AND name like ?");
 	query.addBindValue(iServerNum);
@@ -603,7 +613,7 @@ bool Server::setPW(int id, const QString &pw) {
 
 	hash.addData(pw.toUtf8());
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("UPDATE %1players SET pw=? WHERE server_id = ? AND player_id=?");
 	query.addBindValue(pw.isEmpty() ? QVariant() : QString::fromLatin1(hash.result().toHex()));
 	query.addBindValue(iServerNum);
@@ -623,7 +633,7 @@ bool Server::setEmail(int id, const QString &email) {
 
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("UPDATE %1players SET email=? WHERE server_id = ? AND player_id=?");
 	query.addBindValue(email);
 	query.addBindValue(iServerNum);
@@ -655,7 +665,7 @@ bool Server::setName(int id, const QString &name) {
 
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("UPDATE %1players SET name=? WHERE server_id = ? AND player_id=?");
 	query.addBindValue(name);
 	query.addBindValue(iServerNum);
@@ -686,7 +696,7 @@ bool Server::setTexture(int id, const QByteArray &texture) {
 
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("UPDATE %1players SET texture=? WHERE server_id = ? AND player_id=?");
 	query.addBindValue(tex, QSql::Binary | QSql::In);
 	query.addBindValue(iServerNum);
@@ -703,7 +713,7 @@ void ServerDB::setSUPW(int srvnum, const QString &pw) {
 
 	hash.addData(pw.toUtf8());
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("UPDATE %1players SET pw=? WHERE server_id = ? AND player_id=?");
 	query.addBindValue(QString::fromLatin1(hash.result().toHex()));
 	query.addBindValue(srvnum);
@@ -717,7 +727,7 @@ QString Server::getUserName(int id) {
 		return name;
 
 	TransactionHolder th;
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("SELECT name FROM %1players WHERE server_id = ? AND player_id = ?");
 	query.addBindValue(iServerNum);
 	query.addBindValue(id);
@@ -735,7 +745,7 @@ int Server::getUserID(const QString &name) {
 		return id;
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("SELECT player_id FROM %1players WHERE server_id = ? AND name like ?");
 	query.addBindValue(iServerNum);
 	query.addBindValue(name);
@@ -754,7 +764,7 @@ QByteArray Server::getUserTexture(int id) {
 
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("SELECT texture FROM %1players WHERE server_id = ? AND player_id = ?");
 	query.addBindValue(iServerNum);
 	query.addBindValue(id);
@@ -768,7 +778,7 @@ QByteArray Server::getUserTexture(int id) {
 void Server::addLink(Channel *c, Channel *l) {
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("INSERT INTO %1channel_links (server_id, channel_id, link_id) VALUES (?,?,?)");
 	query.addBindValue(iServerNum);
 	query.addBindValue(c->iId);
@@ -784,7 +794,7 @@ void Server::addLink(Channel *c, Channel *l) {
 void Server::removeLink(Channel *c, Channel *l) {
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 
 	if (l) {
 		SQLPREP("DELETE FROM %1channel_links WHERE server_id = ? AND channel_id = ? AND link_id = ?");
@@ -809,7 +819,7 @@ void Server::removeLink(Channel *c, Channel *l) {
 Channel *Server::addChannel(Channel *p, const QString &name) {
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 
 	SQLPREP("SELECT MAX(channel_id)+1 AS id FROM %1channels WHERE server_id=?");
 	query.addBindValue(iServerNum);
@@ -833,11 +843,12 @@ Channel *Server::addChannel(Channel *p, const QString &name) {
 void Server::removeChannel(const Channel *c) {
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("DELETE FROM %1channels WHERE server_id = ? AND channel_id = ?");
 	query.addBindValue(iServerNum);
 	query.addBindValue(c->iId);
 	SQLEXEC();
+	qhChannels.remove(c->iId);
 }
 
 void Server::updateChannel(const Channel *c) {
@@ -845,7 +856,7 @@ void Server::updateChannel(const Channel *c) {
 	Group *g;
 	ChanACL *acl;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("UPDATE %1channels SET name = ?, parent_id = ?, inheritacl = ? WHERE server_id = ? AND channel_id = ?");
 	query.addBindValue(c->qsName);
 	query.addBindValue(c->cParent ? c->cParent->iId : QVariant());
@@ -882,7 +893,7 @@ void Server::updateChannel(const Channel *c) {
 			query.addBindValue(iServerNum);
 			query.addBindValue(pid);
 			query.addBindValue(1);
-			SQLEXEC();
+			SOFTEXEC();
 		}
 		foreach(pid, g->qsRemove) {
 			SQLPREP("INSERT INTO %1group_members (group_id, server_id, player_id, addit) VALUES (?, ?, ?, ?)");
@@ -890,7 +901,7 @@ void Server::updateChannel(const Channel *c) {
 			query.addBindValue(iServerNum);
 			query.addBindValue(pid);
 			query.addBindValue(0);
-			SQLEXEC();
+			SOFTEXEC();
 		}
 	}
 
@@ -908,7 +919,7 @@ void Server::updateChannel(const Channel *c) {
 		query.addBindValue(acl->bApplySubs ? 1 : 0);
 		query.addBindValue(static_cast<int>(acl->pAllow));
 		query.addBindValue(static_cast<int>(acl->pDeny));
-		SQLEXEC();
+		SOFTEXEC();
 	}
 }
 
@@ -917,7 +928,7 @@ void Server::readChannelPrivs(Channel *c) {
 
 	int cid = c->iId;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("SELECT group_id, name, inherit, inheritable FROM %1groups WHERE server_id = ? AND channel_id = ?");
 	query.addBindValue(iServerNum);
 	query.addBindValue(cid);
@@ -988,25 +999,27 @@ void Server::readChannels(Channel *p) {
 		}
 	}
 
+	query.clear();
+
 	foreach(c, kids)
 	readChannels(c);
 }
 
 void Server::readLinks() {
 	TransactionHolder th;
-	QSqlQuery query;
-	
+	QSqlQuery &query = *th.qsqQuery;
+
 	SQLPREP("SELECT channel_id, link_id FROM %1channel_links WHERE server_id = ?");
 	query.addBindValue(iServerNum);
 	SQLEXEC();
-	
+
 	while (query.next()) {
 		int cid = query.value(0).toInt();
 		int lid = query.value(1).toInt();
-		
+
 		Channel *c = qhChannels.value(cid);
 		Channel *l = qhChannels.value(lid);
-		if (c && l) 
+		if (c && l)
 			c->link(l);
 	}
 }
@@ -1017,13 +1030,13 @@ void Server::setLastChannel(const Player *p) {
 		return;
 
 	TransactionHolder th;
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 
 	SQLPREP("UPDATE %1players SET lastchannel=? WHERE server_id = ? AND player_id = ?");
 	query.addBindValue(p->cChannel->iId);
 	query.addBindValue(iServerNum);
 	query.addBindValue(p->iId);
-	SQLEXEC();
+	SOFTEXEC();
 }
 
 int Server::readLastChannel(int id) {
@@ -1031,7 +1044,7 @@ int Server::readLastChannel(int id) {
 		return 0;
 
 	TransactionHolder th;
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 
 	SQLPREP("SELECT lastchannel FROM %1players WHERE server_id = ? AND player_id = ?");
 	query.addBindValue(iServerNum);
@@ -1080,7 +1093,7 @@ void Server::getBans() {
 
 	qlBans.clear();
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("SELECT base,mask FROM %1bans WHERE server_id = ?");
 	query.addBindValue(iServerNum);
 	SQLEXEC();
@@ -1096,7 +1109,7 @@ void Server::saveBans() {
 	TransactionHolder th;
 	qpBan ban;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("DELETE FROM %1bans WHERE server_id = ? ");
 	query.addBindValue(iServerNum);
 	SQLEXEC();
@@ -1116,7 +1129,7 @@ QVariant Server::getConf(const QString &key, QVariant def) {
 QVariant ServerDB::getConf(int server_id, const QString &key, QVariant def) {
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("SELECT value FROM %1config WHERE server_id = ? AND keystring = ?");
 	query.addBindValue(server_id);
 	query.addBindValue(key);
@@ -1132,7 +1145,7 @@ QMap<QString, QString> ServerDB::getAllConf(int server_id) {
 
 	QMap<QString, QString> map;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("SELECT keystring, value FROM %1config WHERE server_id = ?");
 	query.addBindValue(server_id);
 	SQLEXEC();
@@ -1148,8 +1161,8 @@ void Server::setConf(const QString &key, const QVariant &value) {
 
 void Server::dblog(const char *str) {
 	TransactionHolder th;
-	QSqlQuery query;
-	
+	QSqlQuery &query = *th.qsqQuery;
+
 	// Once per hour
 	if (Meta::mp.iLogDays > 0) {
 		if (ServerDB::tLogClean.isElapsed(3600ULL * 1000000ULL)) {
@@ -1163,7 +1176,7 @@ void Server::dblog(const char *str) {
 			SQLEXEC();
 		}
 	}
-	
+
 	SQLPREP("INSERT INTO %1slog (server_id, msg) VALUES(?,?)");
 	query.addBindValue(iServerNum);
 	query.addBindValue(QLatin1String(str));
@@ -1172,8 +1185,8 @@ void Server::dblog(const char *str) {
 
 QList<QPair<unsigned int, QString> > ServerDB::getLog(int server_id, unsigned int sec_min, unsigned int sec_max) {
 	TransactionHolder th;
-	QSqlQuery query;
-	
+	QSqlQuery &query = *th.qsqQuery;
+
 	QString qstr;
 	if (Meta::mp.qsDBDriver == "QSQLITE") {
 		qstr = QString::fromLatin1("msgtime > datetime('now','-%1 seconds') AND msgtime < datetime('now','-%2 seconds')").arg(sec_max).arg(sec_min);
@@ -1183,9 +1196,9 @@ QList<QPair<unsigned int, QString> > ServerDB::getLog(int server_id, unsigned in
 	ServerDB::prepare(query, QString::fromLatin1("SELECT msgtime, msg FROM %1slog WHERE server_id = ? AND ") + qstr);
 	query.addBindValue(server_id);
 	SQLEXEC();
-	
+
 	QList<QPair<unsigned int, QString> > ql;
-	while(query.next()) {
+	while (query.next()) {
 		QDateTime qdt = query.value(0).toDateTime();
 		QString msg = query.value(1).toString();
 		ql << QPair<unsigned int, QString>(qdt.toTime_t(), msg);
@@ -1196,7 +1209,7 @@ QList<QPair<unsigned int, QString> > ServerDB::getLog(int server_id, unsigned in
 void ServerDB::setConf(int server_id, const QString &key, const QVariant &value) {
 	TransactionHolder th;
 
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	if (value.isNull() || value.toString().trimmed().isEmpty()) {
 		SQLPREP("DELETE FROM %1config WHERE server_id = ? AND keystring = ?");
 		query.addBindValue(server_id);
@@ -1213,7 +1226,7 @@ void ServerDB::setConf(int server_id, const QString &key, const QVariant &value)
 
 QList<int> ServerDB::getAllServers() {
 	TransactionHolder th;
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("SELECT server_id FROM %1servers");
 	SQLEXEC();
 
@@ -1227,7 +1240,7 @@ QList<int> ServerDB::getBootServers() {
 	QList<int> ql = getAllServers();
 
 	TransactionHolder th;
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 
 	QList<int> bootlist;
 	foreach(int i, ql) {
@@ -1243,7 +1256,7 @@ QList<int> ServerDB::getBootServers() {
 
 bool ServerDB::serverExists(int num) {
 	TransactionHolder th;
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("SELECT server_id FROM %1servers WHERE server_id = ?");
 	query.addBindValue(num);
 	SQLEXEC();
@@ -1254,7 +1267,7 @@ bool ServerDB::serverExists(int num) {
 
 int ServerDB::addServer() {
 	TransactionHolder th;
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("SELECT MAX(server_id)+1 AS id FROM %1servers");
 	SQLEXEC();
 	int id = 0;
@@ -1268,7 +1281,7 @@ int ServerDB::addServer() {
 
 void ServerDB::deleteServer(int server_id) {
 	TransactionHolder th;
-	QSqlQuery query;
+	QSqlQuery &query = *th.qsqQuery;
 	SQLPREP("DELETE FROM %1servers WHERE server_id = ?");
 	query.addBindValue(server_id);
 	SQLEXEC();
